#
# Chum::Analysis::Local::Banana
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::Local::Banana - Runs local version of banana

=head1 SYNOPSIS

=head1 DESCRIPTION



=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::Local::Banana;

use vars qw(@ISA @BANANA_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Factory::EMBOSS;
use Chum::Analysis::Local::LocalApplicationI;
use File::Temp qw/ tempfile tempdir unlink0/;

@ISA = qw( Chum::Analysis::Local::LocalApplicationI );
BEGIN {
        @BANANA_PARAMS = qw();
        foreach my $attr (@BANANA_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
	$self->program("banana");
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub run_program {
	my $self = shift;
	my $reference_sequence = shift;
	my $stack_object = shift;
	my $results_file = shift;
	my $meta_input_hashref = shift;

	my $factory = new Bio::Factory::EMBOSS;
	$factory->{'_programs'}->{$self->program} = 1;
	my $banana_application = $factory->program($self->program);
	my ($tmp1fh, $tmp1filename) = tempfile( "banana.tmpXXXXXX", SUFFIX => '.fa', DIR => "/tmp/");
	my %input = (   -sequence => $reference_sequence->sequence,
			-outfile  =>  $results_file);
	$banana_application->run(\%input);
	unlink0($tmp1fh, $tmp1filename);
	return 0;
}

sub convert_output_to_gff {
	my $self = shift;
	my $input_file = shift;
	my $output_file = shift;
	my $seqname = shift;

	open(INFILE, $input_file) || $self->throw("Cannot open input file: $input_file\n");
	my $opened = open(OUTFILE, ">$output_file");
	if (!$opened) {
		close(INFILE);
		$self->throw("Cannot open output file: $output_file\n");
	}

	my $current_seqno = 1;
	foreach my $line (<INFILE>) {
		chomp($line);
		if ($line =~ /([ATCGactgNn]+)[\s]+([\S]+)[\s]+([\S]+)/g) {
			my $nucleotide = $1;
			my $bend = $2;
			my $curve = $3;
			print OUTFILE "$seqname\tBanana\tmisc_feature\t$current_seqno\t$current_seqno\t" . $bend . ":" . $curve . "\t.\t.\n";
			$current_seqno++;
		}
	}
	close(INFILE);
	close(OUTFILE);
}

1;