#
# Chum::Analysis::Local::Dan
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::Local::Dan - Runs local version of dan

=head1 SYNOPSIS

=head1 DESCRIPTION



=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::Local::Dan;

use vars qw(@ISA @DAN_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Factory::EMBOSS;
use Chum::Analysis::Local::LocalApplicationI;
use File::Temp qw/ tempfile tempdir unlink0/;

@ISA = qw( Chum::Analysis::Local::LocalApplicationI );
BEGIN {
        @DAN_PARAMS = qw();
        foreach my $attr (@DAN_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
	$self->program("dan");
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub run_program {
	my $self = shift;
	my $reference_sequence = shift;
	my $stack_object = shift;
	my $results_file = shift;
	my $meta_input_hashref = shift;
	
	my $factory = new Bio::Factory::EMBOSS;
	$factory->{'_programs'}->{$self->program} = 1;
	my $dan_application = $factory->program($self->program);
	my %input = (   -sequence => $reference_sequence->sequence,
			-outfile  =>  $results_file);
	foreach my $key (keys %$meta_input_hashref) {
		if ($key =~ /-[\S]+/g) {
			$input{$key} = $meta_input_hashref->{$key};
		}
	}
	$dan_application->run(\%input);
	return 0;
}

sub convert_output_to_gff {
	my $self = shift;
	my $input_file = shift;
	my $output_file = shift;
	my $seqname = shift;

	open(INFILE, $input_file) || $self->throw("Cannot open input file: $input_file\n");
	my $opened = open(OUTFILE, ">$output_file");
	if (!$opened) {
		close(INFILE);
		$self->throw("Cannot open output file: $output_file\n");
	}
	foreach my $line (<INFILE>) {
		chomp($line);
		if ($line =~ /([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+([ATCGactgNn]+)/g) {
			my $start = $1;
			my $end = $2;
			my $tm = $3;
			my $gc = $4;
			my $deltaG = $5;
			my $deltaH = $6;
			my $deltaS = $7;
			my $tmProd = $8;
			my $sequence = $9;
			print OUTFILE "$seqname\tDAN\tmisc_feature\t$start\t$end\t" . $tm . ":" . $gc . "\t.\t.\n";
		}
	}
	close(INFILE);
	close(OUTFILE);
}

1;