#
# Chum::Analysis::Local::LocalApplicationI
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::Local::LocalApplicationI - Runs a local application interface

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::Local::LocalApplicationI;

use vars qw(@ISA @LOCAL_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Data::Dumper;
use Chum::Analysis::ApplicationI;
use Chum::Result::Result;
use Chum::Result::ResultSet;
use File::Temp qw/ tempfile tempdir unlink0/;

@ISA = qw( Chum::Analysis::ApplicationI );
BEGIN {
        @LOCAL_PARAMS = qw();
        foreach my $attr (@LOCAL_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub binary {
	my $self = shift;
	if(@_) {
		my $binary = shift;
		$self->{'binary'} = $binary;
	}
	return $self->{'binary'};  
}

sub run_program {
	shift->throw_not_implemented();
}

###################################################################################
########## RUN THE VARIATION ANALYSIS
###################################################################################

sub run_variation_analysis {
        my $self = shift;
        my $stack_object = shift;
	my $meta_input_hash = undef;
	if(@_) { $meta_input_hash = shift; } else { my %tmp_hash; $meta_input_hash = \%tmp_hash; }

        if (!defined($stack_object)) { $self->warn("No stack defined"); return undef; }
        elsif (!defined($stack_object->reference_sequence)) { $self->warn("No reference sequence defined"); return undef; }
	elsif (scalar(@{$stack_object->orthologous_sequences}) == 0) { $self->warn("No orthologs"); return undef; }

        my @results = ();
	
        ####RUN THE CANONICAL ANALYSIS
	my (undef, $tmpfilename) = tempfile($self->program . ".tmpXXXXXX", SUFFIX => '.out', DIR => "/tmp/", UNLINK => 1, OPEN => 0);
        my %named_supplementary_result_files_hash;
	my $status = $self->run_program($stack_object->reference_sequence,
					$stack_object,
					$tmpfilename,
					$meta_input_hash,
					\%named_supplementary_result_files_hash);

        if ($status) { $self->warn("WARN: Could not obtain " . $self->program . " results"); return undef; }
        push @results, new Chum::Result::Result('type' => 'CANONICAL',
						'sequence' => $stack_object->reference_sequence,
						'results_file' => $tmpfilename,
						'named_supplementary_result_files' => \%named_supplementary_result_files_hash);
	
        ####RUN THE VARIATION ANALYSES
        foreach my $variant_sequence (@{$stack_object->variant_sequences}) {
		my (undef, $tmpfilename) = tempfile($self->program . ".tmpXXXXXX", SUFFIX => '.out', DIR => "/tmp/", UNLINK => 1, OPEN => 0);
                my %named_supplementary_result_files_hash;
		$status = $self->run_program(  	$variant_sequence,
						$stack_object,
						$tmpfilename,
						$meta_input_hash,
						\%named_supplementary_result_files_hash);

                if ($status) { $self->warn("Could not obtain " . $self->program . " results: variant analysis"); return undef; }
                push @results, new Chum::Result::Result('type' => 'VARIANT',
							'sequence' => $variant_sequence,
							'results_file' => $tmpfilename,
							'named_supplementary_result_files' => \%named_supplementary_result_files_hash);
        }
        ####PUT TOGETHER THE RESULTSET FOR RETURN ANALYSIS
        my $resultset = new Chum::Result::ResultSet(	'analysis_object' => $self,
							'stack' => $stack_object,
							'program' => $self->program,
                                                        'results' => \@results);
        return $resultset;
}
