#
# Chum::Analysis::Local::MatrixScanner
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::Local::MatrixScanner - Runs library-based analysis of variants

=head1 SYNOPSIS

=head1 DESCRIPTION

Uses TFBS::Perl modules (LocalTransfac)

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::Local::MatrixScanner;

use vars qw(@ISA @MSCANNER_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Data::Dumper;
use Chum::Analysis::ApplicationI;
use Chum::Result::Result;
use Chum::Result::ResultSet;

@ISA = qw( Chum::Analysis::ApplicationI );
BEGIN {
        @MSCANNER_PARAMS = qw(matrix_db);
        foreach my $attr (@MSCANNER_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
	if (!defined($self->{'matrix_db'})) {
		$self->throw("matrix_db not defined");
	} elsif (!$self->{'matrix_db'}->isa("Chum::DB::MatrixDB")) {
		$self->throw("matrix_db is not a Chum::DB::MatrixDB object");	
	} 
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

###################################################################################
########## RUN THE VARIATION ANALYSIS FOR THE MATRIX DATABASE (IT'S LOADED MATRIXSET)
###################################################################################

sub run_variation_analysis {
        my $self = shift;
        my $stack_object = shift;
	my @results;

	#####################REFERENCE ANALYSIS############################
	my $ref_siteset = $self->{'matrix_db'}->_detect($stack_object->reference_sequence->sequence->seq);
	my @ref_sites =();
	my $ref_siteset_iterator = $ref_siteset->Iterator(-sort_by =>'score');
	while (my $site = $ref_siteset_iterator->next) {
		push @ref_sites, $site;
	}
	push @results, new Chum::Result::Result('type' => 'CANONICAL',
						'sequence' => $stack_object->reference_sequence,
						'results_object' => \@ref_sites);

	#####################VARIANT ANALYSIS############################
	foreach my $variant_sequence (@{$stack_object->variant_sequences}) {
		my $var_siteset = $self->{'matrix_db'}->_detect($variant_sequence->sequence->seq);
		my @var_sites =();
		my $var_siteset_iterator = $var_siteset->Iterator(-sort_by =>'score');
		while (my $site = $var_siteset_iterator->next) {
			push @var_sites, $site;
		}
		push @results, new Chum::Result::Result('type' => 'VARIANT',
							'sequence' => $variant_sequence,
							'results_object'  => \@var_sites);
	}

	#####################RETURN RESULTSET############################
        my $resultset = new Chum::Result::ResultSet(	'analysis_object' => $self,
							'stack' => $stack_object,
							'program' => "MATRIXSCANNER",
                                                        'results' => \@results);
	return $resultset;
}
