#
# Chum::Analysis::Local::Transfac
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::Local::Transfac - Runs library-based analysis of variants

=head1 SYNOPSIS

=head1 DESCRIPTION

Uses TFBS::Perl modules (LocalTransfac)

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::Local::Transfac;

use vars qw(@ISA @TRANSFAC_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Data::Dumper;
use Chum::Analysis::ApplicationI;
use Chum::Result::Result;
use Chum::Result::ResultSet;
require TFBS::DB::LocalTRANSFAC;

@ISA = qw( Chum::Analysis::ApplicationI );
BEGIN {
        @TRANSFAC_PARAMS = qw(transfac_dir _transfac_db);
        foreach my $attr (@TRANSFAC_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
	if (defined($self->{'transfac_dir'})) {
		$self->{'_transfac_db'} = TFBS::DB::LocalTRANSFAC->connect(	-localdir => $self->{'transfac_dir'},
										-accept_conditions => 1);
		$self->{'_matrix_set'} = $self->{'_transfac_db'}->get_all_Matrices('PWM');
	}
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

###################################################################################
########## RUN THE VARIATION ANALYSIS FOR TRANSFAC
###################################################################################

sub run_variation_analysis {
        my $self = shift;
        my $tstack_obj = shift;
	my @results;

	#####################REFERENCE ANALYSIS############################
	my $ref_siteset = $self->_detect($tstack_obj->reference_sequence->sequence->seq);
	my @ref_sites =();
	my $ref_siteset_iterator = $ref_siteset->Iterator(-sort_by =>'score');
	while (my $site = $ref_siteset_iterator->next) {
		print STDERR $site->score . "\t" . $site->primary_tag . "\t" . $site->source_tag . "\t" . $site->pattern->name . "\n";
		push @ref_sites, $site;
	}
	push @results, new Chum::Result::Result('type' => 'CANONICAL',
						'sequence' => $tstack_obj->reference_sequence,
						'results_file' => undef,
						'results_object' => \@ref_sites);

	#####################VARIANT ANALYSIS############################
	foreach my $variant_sequence (@{$tstack_obj->variant_sequences}) {
		my $var_siteset = $self->_detect($variant_sequence->sequence->seq);
		my @var_sites =();
		my $var_siteset_iterator = $var_siteset->Iterator(-sort_by =>'score');
		while (my $site = $var_siteset_iterator->next) {
			push @var_sites, $site;
		}
		push @results, new Chum::Result::Result('type' => 'VARIANT',
							'sequence' => $variant_sequence,
							'results_file' => undef,
							'results_object'  => \@var_sites);
	}

	#####################RETURN RESULTSET############################
        my $resultset = new Chum::Result::ResultSet(	'analysis_object' => $self,
							'stack' => $tstack_obj,
							'program' => "TRANSFAC",
                                                        'results' => \@results);
	return $resultset;
}

sub destroy {
	my $self = shift;
	$self->{'_matrix_set'} = undef;
}

sub _find_pwm_in_transfac {
        my $self = shift;
	my $matrix_name = shift;
        $self->{'matrix'} = $self->{'_transfac_db'}->get_Matrix_by_name($matrix_name, 'PWM');
        return $self->{'matrix'};
}

sub _detect {
        my $self = shift;
        my $sequence = shift;
        my $siteset = $self->{'_matrix_set'}->search_seq(	-seqstring => $sequence,
								-threshold => "80%");
        return $siteset;
}
