#
# Chum::Result::Interpretter::Interpretter
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Result::Interpretter::InterpretterI - Interpret result sets

=head1 SYNOPSIS

=head1 DESCRIPTION

Interpret result sets

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut

package Chum::Result::Interpretter::InterpretterI;

use vars qw(@ISA @INTERPRETTERI_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @INTERPRETTERI_PARAMS = qw(resultset);
        foreach my $attr (@INTERPRETTERI_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->$attr($value);
        }
	if (!defined($self->{'resultset'})) { $self->throw("ERROR: Missing Chum::Result::ResultSet object"); }
        $self->_preload_data();
	return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub resultset {
	my $self = shift;
	if(@_) {
		my $resultset = shift;
		if (!$resultset->isa("Chum::Result::ResultSet")) {
			$self->throw("ERROR: resultset must be a Chum::Result::ResultSet object");
		}
		$self->{'resultset'} = $resultset;
	}
	return $self->{'resultset'};
}

sub _preload_data() {
	shift->throw_not_implemented();	
}

1;
