#
# Chum::Result::MotifPrediction::Matrix
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Result::MotifPrediction::Matrix - A matrix from a motif prediction analysis run

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Result::MotifPrediction::Matrix;

use vars qw(@ISA @M_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Chum::Result::MotifPrediction::MatrixIndex;

@ISA = qw( Bio::Root::Root );
BEGIN {
    @M_PARAMS = qw(name type indices);
    foreach my $attr (@M_PARAMS)
            { $OK_FIELD{$attr}++; }
}

sub new {
    my($class, @args) = @_;
    my $self = $class->SUPER::new(@args);
    while (@args) {
        my $attr = shift @args;
        my $value = shift @args;
        $self->{$attr} = $value;
    }
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub name {
    my $self = shift;
    return $self->{'name'}; 
}

sub type {
    my $self = shift;
    return $self->{'type'};  
}

sub indices {
    my $self = shift;
    return $self->{'indices'};   
}

sub load {
    my $self = shift;
    my $m_node = shift;
    if (!defined($m_node)) { $self->throw("No XML::Twig::Elt object defined for loading"); }
    unless ($m_node->isa("XML::Twig::Elt")) { $self->throw("Loading object is not a XML::Twig::Elt object"); }
    
    $self->{'name'} = $m_node->att('name');
    $self->{'type'} = $m_node->att('type');
    
    my @indices = ();
    foreach my $i_node ($m_node->children('index')){
        my $matrix_index = new Chum::Result::MotifPrediction::MatrixIndex();
        $matrix_index->load($i_node);
        push @indices, $matrix_index;
    }
    $self->{'indices'} = \@indices;   
}

1;