#
# Chum::Result::MotifPrediction::PredictedMotif
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Result::MotifPrediction::PredictedMotif - An predicted motif from a motif prediction analysis run

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Result::MotifPrediction::PredictedMotif;

use vars qw(@ISA @PM_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Chum::Result::MotifPrediction::Matrix;
use Chum::Result::MotifPrediction::Hit;

@ISA = qw( Bio::Root::Root );
BEGIN {
    @PM_PARAMS = qw(score sequence matrices hits);
    foreach my $attr (@PM_PARAMS)
            { $OK_FIELD{$attr}++; }
}

sub new {
    my($class, @args) = @_;
    my $self = $class->SUPER::new(@args);
    while (@args) {
            my $attr = shift @args;
            my $value = shift @args;
            $self->{$attr} = $value;
    }
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub score {
    my $self = shift;
    return $self->{'score'}; 
}

sub sequence {
    my $self = shift;
    return $self->{'sequence'};  
}

sub matrices {
    my $self = shift;
    return $self->{'matrices'};   
}

sub hits {
    my $self = shift;
    return $self->{'hits'};   
}

sub load {
    my $self = shift;
    my $pm_node = shift;
    if (!defined($pm_node)) { $self->throw("No XML::Twig::Elt object defined for loading"); }
    unless ($pm_node->isa("XML::Twig::Elt")) { $self->throw("Loading object is not a XML::Twig::Elt object"); }
    
    $self->{'score'} = $pm_node->att('score');
    $self->{'sequence'} = $pm_node->att('seq');
    
    my @matrices = ();
    foreach my $m_node ($pm_node->children('matrix')){
        my $matrix = new Chum::Result::MotifPrediction::Matrix();
        $matrix->load($m_node);
        push @matrices, $matrix;
    }
    $self->{'matrices'} = \@matrices;
    
    my @hits = ();
    foreach my $h_node ($pm_node->children('hit')){
        my $hit = new Chum::Result::MotifPrediction::Hit();
        $hit->load($h_node);
        push @hits, $hit;
    }
    $self->{'hits'} = \@hits;
}

1;