#
# Chum::Statistics::AnalysisResult
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod
=head1 NAME
Chum::Statistics::AnalysisResult - A results files object
=head1 SYNOPSIS
=head1 DESCRIPTION
=head1 CONTACT
Stephen Montgomery (smontgom@bcsgc.bc.ca)
=head1 METHODS
=cut

package Chum::Statistics::AnalysisResult;

use vars qw(@ISA @ANALYSISR_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Bio::Seq;
use Data::Dumper;
use Chum::VariantSequence;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @ANALYSISR_PARAMS = qw(file);
        foreach my $attr (@ANALYSISR_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
        if (!defined($self->{'file'})) { $self->throw("No 'file' specified"); }
        unless (-e $self->{'file'}) { $self->throw("'file' does not exist"); } 
        $self->_load_results_file();    
        if ($self->{'file'} =~ /(rs[0-9]+)/) {
            $self->{'id'} = $1;
        } elsif ($self->{'file'} =~ /(OREG[0-9]+)/) {
            $self->{'id'} = $1;   
        }
        
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub _load_results_file {
    my $self = shift;
    my ($results1, $results2, $results3, $results4, $results5, $results6, $results7, $results8, $results9, $results10, $results11, $results12, $results13, $results14, $results15, $results16, $results17, $results18);
    open(INPUT, $self->{'file'}) || die("Cannot open existing output file for input\n");
    print STDERR "Loading file: " . $self->{'file'} . "\n";
    my $run_number;
    my $run_line;
    my $run_status;
    foreach my $line (<INPUT>) {
            chomp($line);
            if ($line =~ /[\S]{1}TEST([0-9]+):[\S]+:([\S]+)/g || $line =~ /[\S]{1}(DONE)/g) {
                    if (defined($run_number)) {
                            if ($run_number == 1) { $results1 = $run_line; }
                            elsif ($run_number == 2) { $results2 = $run_line; }
                            elsif ($run_number == 3) { $results3 = $run_line; }
                            elsif ($run_number == 4) { $results4 = $run_line; }
                            elsif ($run_number == 5) { $results5 = $run_line; }
                            elsif ($run_number == 6) { $results6 = $run_line; }
                            elsif ($run_number == 7) { $results7 = $run_line; }
                            elsif ($run_number == 8) { $results8 = $run_line; }
                            elsif ($run_number == 9) { $results9 = $run_line; }
                            elsif ($run_number == 10) { $results10 = $run_line; }
			    elsif ($run_number == 11) { $results11 = $run_line; }
			    elsif ($run_number == 12) { $results12 = $run_line; }
                            elsif ($run_number == 13) { $results13 = $run_line; }
                            elsif ($run_number == 14) { $results14 = $run_line; }
                            elsif ($run_number == 15) { $results15 = $run_line; }
                            elsif ($run_number == 16) { $results16 = $run_line; }
                            elsif ($run_number == 17) { $results17 = $run_line; }
			    elsif ($run_number == 18) { $results18 = $run_line; }
			    
                    }
                    $run_number = $1;
                    $run_line = $line . "\n";
                    $run_status = $2;
                    if ($run_number eq "DONE") { last; }
            } elsif (defined($run_number)) {
                    $run_line .= $line . "\n";
            }
    }
    close(INPUT);
    $self->_loadTransfacTestResults($results1);
    $self->_loadWeederTestResults($results2);
    $self->_loadMotifsamplerTestResults($results3);
    $self->_loadBananaTestResults($results4);
    $self->_loadDanTestResults($results5);
    $self->_loadDistanceTestResults($results6);
    $self->_loadKaksTestResults($results7);
    $self->_loadRepeatTestResults($results8);
    $self->_loadAlleleFrequencyTestResults($results9);
    $self->_loadDerivedAlleleFrequencyTestResults($results10);
    $self->_loadTransitionTransversionResults($results11);
    $self->_loadCoexpressionJasparTestResults($results12);
    $self->_loadCpGIslandTestResults($results13);
    $self->_loadNhgriDNaseTestResults($results14);
    $self->_loadNobleDNaseTestResults($results15);
    $self->_loadPhastConsTestResults($results16);
    $self->_loadRegulatoryPotentialTestResults($results17);
    $self->_loadAlignmentDistanceTestResults($results18);
}


sub isNotSNP {
        my $self = shift;
        #if (!defined($self->{'bend'}->{'score'})) {
        #        return 1;
        #} elsif ( length($self->{'bend'}->{'score'}) == 0) {
        #        return 1;
        #} elsif ($self->{'bend'}->{'score'} eq "UNDEFINED") {
        #        return 1;
        if (!defined($self->{'transition-transversion'})) {
                return 1;
        } elsif (length($self->{'transition-transversion'}) == 0) {
                return 1;
        } elsif ($self->{'transition-transversion'} eq "0") {
                return 1;
        } else {
                return 0;       
        }
}

sub _loadTransfacTestResults {
    my $self = shift;
    my $results = shift;
    my @lines = split (/\n/, $results);
    if (scalar(@lines) == 2) {
        if ($lines[1] =~ /([\S]+)[\s]*/g) {
            $self->{'transfac'}->{'score'} = $1;
        } else {
	    $self->{'transfac'}->{'score'} = "UNDEFINED";
	}
    } else {
        $self->{'transfac'}->{'score'} = "UNDEFINED";    
    } 
}

sub _loadWeederTestResults {
    my $self = shift;
    my $results = shift;
    my @lines = split (/\n/, $results);
    if (scalar(@lines) == 3) {
        if ($lines[1] =~ /VAR:([\S]+)[\s]*/g) {
            $self->{'weeder'}->{'score'} = $1;
        } else {
	    $self->{'weeder'}->{'score'} = "UNDEFINED";
	}
	if ($lines[2] =~ /MAX:([\S]+)[\s]*/g) {
            $self->{'weeder'}->{'max_score'} = $1;
        } else {
            $self->{'weeder'}->{'max_score'} = "UNDEFINED";
        }
    } else {
        $self->{'weeder'}->{'score'} = "UNDEFINED";
	$self->{'weeder'}->{'max_score'} = "UNDEFINED";
    } 
}

sub _loadMotifsamplerTestResults {
    my $self = shift;
    my $results = shift;
    my @lines = split (/\n/, $results);
    if (scalar(@lines) == 3) {
        if ($lines[1] =~ /VAR:([\S]+)[\s]*/g) {
            $self->{'motifsampler'}->{'score'} = $1;
        } else {
	    $self->{'motifsampler'}->{'score'} = "UNDEFINED";
        }
	if ($lines[2] =~ /MAX:([\S]+)[\s]*/g) {
            $self->{'motifsampler'}->{'max_score'} = $1;
        } else {
            $self->{'motifsampler'}->{'max_score'} = "UNDEFINED";
        }
    } else {
        $self->{'motifsampler'}->{'score'} = "UNDEFINED";
	$self->{'motifsampler'}->{'max_score'} = "UNDEFINED";
    } 
}

sub _loadBananaTestResults {
    my $self = shift;
    my $results = shift;
    my @lines = split (/\n/, $results);
    if (scalar(@lines) == 3) {
        if ($lines[1] =~ /BEND:([\S]+)[\s]*/g) {
            $self->{'bend'}->{'score'} = $1;
        }
        if ($lines[2] =~ /CURVE:([\S]+)[\s]*/g) {
            $self->{'curve'}->{'score'} = $1;   
        }
    } else {
        $self->{'bend'}->{'score'} = "UNDEFINED";
        $self->{'curve'}->{'score'} = "UNDEFINED";
    } 
}

sub _loadDanTestResults {
    my $self = shift;
    my $results = shift;
    my @lines = split (/\n/, $results);
    if (scalar(@lines) == 3) {
        if ($lines[1] =~ /GC:([\S]+)[\s]*/g) {
            $self->{'gc'}->{'score'} = $1;
        }
        if ($lines[2] =~ /TM:([\S]+)[\s]*/g) {
            $self->{'temperature'}->{'score'} = $1;   
        }
    } else {
        $self->{'gc'}->{'score'} = "UNDEFINED";
        $self->{'temperature'}->{'score'} = "UNDEFINED";
    } 
}

sub _loadDistanceTestResults {
    my $self = shift;
    my $results = shift;
    my @lines = split (/\n/, $results);
    if (scalar(@lines) >= 2) {
        for (my $i = 1; $i < scalar(@lines); $i++) {
            if ($lines[$i] =~ /([\S]+):([\S]+):([\S\s]+):([0-9]+):([\S]+)/g) {
                $self->{'distance'}->{$2}->{'gene'} = $1;
                $self->{'distance'}->{$2}->{'external_gene'} = $3;
                $self->{'distance'}->{$2}->{'distance'} = $4;
		$self->{'distance'}->{$2}->{'ln_distance'} = $5;
            }
        }
    } else {
        $self->{'distance'} = 0;
    }
}

sub _loadKaksTestResults {
    my $self = shift;
    my $results = shift;
    my @lines = split (/\n/, $results);
    if (scalar(@lines) >= 2) {
        for (my $i = 1; $i < scalar(@lines); $i++) {
            if ($lines[$i] =~ /([\S]+):([\S\s]+):([\S\s]+):([\S]+)/g) {
                $self->{'kaks'}->{$1}->{$2}->{$3} = $4;
            }
        } 
    } else {
        $self->{'kaks'} = 0;
    }
}

sub _loadRepeatTestResults {
    my $self = shift;
    my $results = shift;
    my @lines = split (/\n/, $results);
    if (scalar(@lines) >= 2) {
        my $repeat_length = 0;
        for (my $i = 1; $i < scalar(@lines); $i++) {
            if ($lines[$i] =~ /LENGTH:([\S\s]+)/g) {
                $self->{'repeat'}->{'search_length'} = $1;
	    } elsif ($lines[$i] =~ /VAR_IN_REPEAT:([\S\s]+)/g) {
		$self->{'repeat'}->{'in_repeat'} = $1;
            } elsif ($lines[$i] =~ /LONG_EVENTS:([\S\s]+)/g) {
		$self->{'repeat'}->{'long_events'} = $1;
            } elsif ($lines[$i] =~ /SHORT_EVENTS:([\S\s]+)/g) {
		$self->{'repeat'}->{'short_events'} = $1;
            } elsif ($lines[$i] =~ /([\S\s]+):([\S\s]+)/g) {
                $self->{'repeat'}->{'repeats'}->{$1} = $2;
                $repeat_length += $2;
            }
        }
        $self->{'repeat'}->{'repeat_length'} = $repeat_length;
    } else {
        $self->{'repeat'} = 0;
    }
}

sub _loadAlleleFrequencyTestResults {
    my $self = shift;
    my $results = shift;
    my @lines = split (/\n/, $results);
    if (scalar(@lines) >= 2) {
        for (my $i = 1; $i < scalar(@lines); $i++) {
            if ($lines[$i] =~ /([\S\s]+):([\S\s]+):([\S\s]+):([\S\s]+)/g) {
                my $population = $1;
                if (!defined($self->{'allele_frequency'}->{$population})) {
                    my @alleles = ();
                    my %allele_frequency_hashref = ('alleles' => $3,
						    'samples' => $2,
				                    'frequency' => $4);
                    push @alleles, \%allele_frequency_hashref;
                    $self->{'allele_frequency'}->{$population} = \@alleles;
                } else {
                    my @alleles = @{$self->{'allele_frequency'}->{$population}};
                    my %allele_frequency_hashref = ('alleles' => $3,
						    'samples' => $2,
				                    'frequency' => $4);
                    push @alleles, \%allele_frequency_hashref;
                    $self->{'allele_frequency'}->{$population} = \@alleles;   
                }
            }
        }
        if (defined($self->{'allele_frequency'}) && $self->{'allele_frequency'} != 0) {
            my $population_hashref = $self->{'allele_frequency'};
            foreach my $population (keys %$population_hashref) {
                my @alleles = @{$self->{'allele_frequency'}->{$population}};
                my $minor_allele_frequency = undef;
                ##sort alleles by frequency, first allele below 0.5 is MAF here
                if (scalar(@alleles) == 2) {
                    foreach my $allele (@alleles) {
                        if (!defined($minor_allele_frequency)) {
                            $minor_allele_frequency = $allele; 
                        } else {
                                if ($allele->{'frequency'} ne "VR") {
                                    if ($minor_allele_frequency->{'frequency'} > $allele->{'frequency'}) {
                                        $minor_allele_frequency = $allele;   
                                    }
                                }
                        }
                    }
                    $self->{'minor_allele_frequency'}->{$population} = $minor_allele_frequency;
                
                } else {
                    #Ignore tri-allelic mafs
                    $self->{'allele_frequency'}->{$population} = undef;
                    $self->{'minor_allele_frequency'}->{$population} = undef;  
                }
            }
        } else {
            $self->{'allele_frequency'} = 0;
            $self->{'minor_allele_frequency'} = 0;
        }
    } else {
        $self->{'allele_frequency'} = 0;
        $self->{'minor_allele_frequency'} = 0;
    }
}

sub _loadDerivedAlleleFrequencyTestResults {
    my $self = shift;
    my $results = shift;
    my @lines = split (/\n/, $results);
    if (scalar(@lines) >= 2) {
        for (my $i = 1; $i < scalar(@lines); $i++) {
            if ($lines[$i] =~ /DAF:([\S\s]+):([\S\s]+)/g) {
                my $population =  $1;
                my $frequency = $2;
                $self->{'daf'}->{$population} = $frequency;
            }
        }
        if (!defined($self->{'daf'})) {
            $self->{'daf'} = "UNDEFINED";     
        }
    } else {
        $self->{'daf'} = "UNDEFINED";    
    } 
}

sub _loadMonthamTestResults {
    my $self = shift;   
    my $results = shift;
    my @lines = split (/\n/, $results);
    if (scalar(@lines) >= 2) {
        for (my $i = 1; $i < scalar(@lines); $i++) {
            if ($lines[$i] =~ /([\S\s]+):([\S\s]+):([\S\s]+):([\S\s]+)/g) {
                $self->{'montham'}->{$1}->{'variant_sequence'} = $2;
                $self->{'montham'}->{$1}->{'reference_mps'} = $3;
                $self->{'montham'}->{$1}->{'variant_mps'} = $4;
            }
        }  
    } else {
        $self->{'montham'} = 0;
    }
}

sub _loadTransitionTransversionResults {
	my $self = shift;
	my $results = shift;
	my @lines = split (/\n/, $results);
	if (scalar(@lines) >= 2) {
        	for (my $i = 1; $i < scalar(@lines); $i++) {
			$self->{'transition-transversion'} = $lines[$i];
		}
	} else {
        	$self->{'transition-transversion'} = 0;
    	}
}

sub _loadCoexpressionJasparTestResults {
	my $self = shift;
    	my $results = shift;
    	my @lines = split (/\n/, $results);
    	if (scalar(@lines) == 2) {
        	if ($lines[1] =~ /([\S]+)[\s]*/g) {
            		$self->{'coexpression-jaspar'}->{'score'} = $1;
        	} else {
            		$self->{'coexpression-jaspar'}->{'score'} = "UNDEFINED";
        	}
    	} else {
        	$self->{'coexpression-jaspar'}->{'score'} = "UNDEFINED";
    	}
}

sub _loadCpGIslandTestResults {
	my $self = shift;
        my $results = shift;
        my @lines = split (/\n/, $results);
        if (scalar(@lines) == 2) {
                if ($lines[1] =~ /([\S]+)[\s]*/g) {
                        $self->{'cpg'}->{'score'} = $1;
                } else {
                        $self->{'cpg'}->{'score'} = "UNDEFINED";
                }
        } else {
                $self->{'cpg'}->{'score'} = "UNDEFINED";
        }
}

sub _loadNhgriDNaseTestResults {
        my $self = shift;
        my $results = shift;
        my @lines = split (/\n/, $results);
        if (scalar(@lines) == 2) {
                if ($lines[1] =~ /([\S]+)[\s]*/g) {
                        $self->{'nhgri-dnase'}->{'score'} = $1;
                } else {
                        $self->{'nhgri-dnase'}->{'score'} = "UNDEFINED";
                }
        } else {
                $self->{'nhgri-dnase'}->{'score'} = "UNDEFINED";
        }
}

sub _loadNobleDNaseTestResults {
	my $self = shift;
        my $results = shift;
        my @lines = split (/\n/, $results);
        if (scalar(@lines) == 2) {
                if ($lines[1] =~ /([\S]+)[\s]*/g) {
                        $self->{'noble-dnase'}->{'score'} = $1;
                } else {
                        $self->{'noble-dnase'}->{'score'} = "UNDEFINED";
                }
        } else {
                $self->{'noble-dnase'}->{'score'} = "UNDEFINED";
        }
}

sub _loadPhastConsTestResults {
	my $self = shift;
        my $results = shift;
        my @lines = split (/\n/, $results);
        if (scalar(@lines) == 2) {
                if ($lines[1] =~ /([\S]+)[\s]*/g) {
                        $self->{'phastcons'}->{'score'} = $1;
                } else {
                        $self->{'phastcons'}->{'score'} = "UNDEFINED";
                }
        } else {
                $self->{'phastcons'}->{'score'} = "UNDEFINED";
        }
}

sub _loadRegulatoryPotentialTestResults {
	my $self = shift;
        my $results = shift;
        my @lines = split (/\n/, $results);
        if (scalar(@lines) == 2) {
                if ($lines[1] =~ /([\S]+)[\s]*/g) {
                        $self->{'rp'}->{'score'} = $1;
                } else {
                        $self->{'rp'}->{'score'} = "UNDEFINED";
                }
        } else {
                $self->{'rp'}->{'score'} = "UNDEFINED";
        }
}

sub _loadAlignmentDistanceTestResults {
    my $self = shift;
    my $results = shift;
    my @lines = split (/\n/, $results);
    if (scalar(@lines) == 2) {
    	if ($lines[1] =~ /([\S]+)[\s]*/g) {
        	$self->{'alignment_distance'} = $1;
        } else {
        	$self->{'alignment_distance'} = "UNDEFINED";
	}
    } else {
        $self->{'alignment_distance'} = "UNDEFINED";
    }
}

1;
