#
# Chum::Statistics::Gist::GistUtils
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod
=head1 NAME
Chum::Statistics::Gist::GistUtils - Gist file utilities
=head1 SYNOPSIS
=head1 DESCRIPTION
=head1 CONTACT
Stephen Montgomery (smontgom@bcsgc.bc.ca)
=head1 METHODS
=cut

package Chum::Statistics::Gist::GistUtils;

use vars qw(@ISA @GUTILS_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );
BEGIN {
    @GUTILS_PARAMS = qw();
    foreach my $attr (@GUTILS_PARAMS)
        { $OK_FIELD{$attr}++; }
}

sub new {
    my($class, @args) = @_;
    my $self = $class->SUPER::new(@args);
    while (@args) {
        my $attr = shift @args;
        my $value = shift @args;
        $self->{$attr} = $value;
    }
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub getMappingOfScoreAndRocFiles() {
    my $self = shift;   
    my $score_files_arrayref = shift;
    my $roc_files_arrayref = shift;
    
    if (!defined($score_files_arrayref)) { $self->throw("No score file arrayref provided as arg1"); }
    if (!defined($roc_files_arrayref)) { $self->throw("No ROC file arrayref provided as arg2"); }
    
    my @score_files = @$score_files_arrayref;
    my @roc_files = @$roc_files_arrayref;
    
    my %SCORE_TYPE_PARTITION_HASH;
    my %SCORE_MATCH_HASH;
    foreach my $score_file (@score_files) {
        if ($score_file =~ /[\S\s]+partition.(test|train).([\S]+).([0-9]+).score/g) {
            my $type_name = $2;
            my $partition_number = $3;
            $SCORE_TYPE_PARTITION_HASH{$partition_number}{$type_name} = $score_file;
            $SCORE_MATCH_HASH{$score_file} = 0;
        } else {
            $self->throw("Unknown score file naming convention: " . $score_file);   
        }
    }
    
    my @PAIRINGS;
    my @UNMATCHED = ();
    foreach my $roc_file (@roc_files) {
        if ($roc_file =~ /[\S\s]+partition.(test|train).([\S]+).([0-9]+).roc/g) {  
            my $type_name = $2;
            my $partition_number = $3;
            if (defined($SCORE_TYPE_PARTITION_HASH{$partition_number}{$type_name})) {
                my $score_file = $SCORE_TYPE_PARTITION_HASH{$partition_number}{$type_name};
                my %dublet = ('score' => $score_file, 'roc' => $roc_file);
                push @PAIRINGS, \%dublet;
                $SCORE_MATCH_HASH{$score_file} = 1;
            } else {
                push @UNMATCHED, $roc_file;
            }
        } else {
            $self->throw("Unknown ROC file naming convention: " . $roc_file);   
        }
    }
    
    if (scalar(@UNMATCHED) > 0) {
        foreach my $unmatched_file (@UNMATCHED) {
            $self->warn("Could not match file: " .  $unmatched_file);  
        }
    }
    foreach my $key (keys %SCORE_MATCH_HASH) {
        if (!$SCORE_MATCH_HASH{$key}) {
            $self->warn("Could not match file: " .  $key);  
        }
    }
    return \@PAIRINGS;
}