package Chum::Utils::ExpectancyManager;

use vars qw(@ISA @EM_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Chum::Utils::ExpectancyLoader;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );

BEGIN {
	@EM_PARAMS = qw(expectancy_loader e_cpg e_noble e_nhgri e_phastcons e_rp e_repeat);
	foreach my $attr (@EM_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
	while (@args) {
		my $attr = shift @args;
		my $value = shift @args;
		$self->$attr($value);
	}
	$self->{'expectancy_loader'} = new Chum::Utils::ExpectancyLoader();
	return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub get_in_cpg_expectancy_value {
	my $self = shift;
	my $chromosome = shift;
	my $distance_value = shift;
	if (!defined($self->{'e_cpg'})) {
		$self->{'e_cpg'} = $self->{'expectancy_loader'}->get_cpg_expectancies();	
	}
	my $values_hashref = $self->{'e_cpg'}->{$chromosome}->{'all'};
	my @keys = sort {$b <=> $a} keys (%$values_hashref);
	my $size = scalar(@keys);
	my $value = 0;
	for(my $i = 0; $i < $size; $i++) {
		if ($keys[$i] == $distance_value) {
			$value = $values_hashref->{$keys[$i]};
			last;
		} elsif (($i+1) != $size) {
			if ($distance_value < $keys[$i] && $distance_value > $keys[$i+1]) {
				$value = ((($values_hashref->{$keys[$i]}  - $values_hashref->{$keys[$i+1]}) / abs( $keys[$i+1] - $keys[$i])) * ($distance_value - $keys[$i])) + $values_hashref->{$keys[$i]};
				last;
			}
		} else {
			warn("Out of range of expectancy values");	
		}
	}
	return $value;
}

sub get_nhgri_expectancy_value {
	my $self = shift;
	my $chromosome = shift;
	my $distance_value = shift;
	if (!defined($self->{'e_nhgri'})) {
		$self->{'e_nhgri'} = $self->{'expectancy_loader'}->get_nhgri_expectancies();	
	}
	my $values_hashref = $self->{'e_nhgri'}->{$chromosome}->{'all'};
	my @keys = sort {$b <=> $a} keys (%$values_hashref);
	my $size = scalar(@keys);
	my $value = 0;
	for(my $i = 0; $i < $size; $i++) {
		if ($keys[$i] == $distance_value) {
			$value = $values_hashref->{$keys[$i]};
			last;
		} elsif (($i+1) != $size) {
			if ($distance_value < $keys[$i] && $distance_value > $keys[$i+1]) {
				$value = ((($values_hashref->{$keys[$i]}  - $values_hashref->{$keys[$i+1]}) / abs( $keys[$i+1] - $keys[$i])) * ($distance_value - $keys[$i])) + $values_hashref->{$keys[$i]};
				last;
			}
		} else {
			warn("Out of range of expectancy values");	
		}
	}
	return $value;
}

sub get_noble_expectancy_value {
	my $self = shift;
	my $chromosome = shift;
	my $distance_value = shift;
	if (!defined($self->{'e_noble'})) {
		$self->{'e_noble'} = $self->{'expectancy_loader'}->get_noble_expectancies();	
	}
	my $values_hashref = $self->{'e_noble'}->{$chromosome}->{'all'};
	my @keys = sort {$b <=> $a} keys (%$values_hashref);
	my $size = scalar(@keys);
	my $value = 0;
	for(my $i = 0; $i < $size; $i++) {
		if ($keys[$i] == $distance_value) {
			$value = $values_hashref->{$keys[$i]};
			last;
		} elsif (($i+1) != $size) {
			if ($distance_value < $keys[$i] && $distance_value > $keys[$i+1]) {
				$value = ((($values_hashref->{$keys[$i]}  - $values_hashref->{$keys[$i+1]}) / abs( $keys[$i+1] - $keys[$i])) * ($distance_value - $keys[$i])) + $values_hashref->{$keys[$i]};
				last;
			}
		} else {
			warn("Out of range of expectancy values");	
		}
	}
	return $value;
}

sub get_phastcons_expectancy_value {
	my $self = shift;
	my $chromosome = shift;
	my $distance_value = shift;
	if (!defined($self->{'e_phastcons'})) {
		$self->{'e_phastcons'} = $self->{'expectancy_loader'}->get_phastcons_expectancies();	
	}
	my $values_hashref = $self->{'e_phastcons'}->{$chromosome}->{'all'};
	my @keys = sort {$b <=> $a} keys (%$values_hashref);
	my $size = scalar(@keys);
	my $value = 0;
	for(my $i = 0; $i < $size; $i++) {
		if ($keys[$i] == $distance_value) {
			$value = $values_hashref->{$keys[$i]};
			last;
		} elsif (($i+1) != $size) {
			if ($distance_value < $keys[$i] && $distance_value > $keys[$i+1]) {
				$value = ((($values_hashref->{$keys[$i]}  - $values_hashref->{$keys[$i+1]}) / abs( $keys[$i+1] - $keys[$i])) * ($distance_value - $keys[$i])) + $values_hashref->{$keys[$i]};
				last;
			}
		} else {
			warn("Out of range of expectancy values");	
		}
	}
	return $value;
}

sub get_regulatory_potential_expectancy_value {
	my $self = shift;
	my $chromosome = shift;
	my $distance_value = shift;
	if (!defined($self->{'e_rp'})) {
		$self->{'e_rp'} = $self->{'expectancy_loader'}->get_rp_expectancies();	
	}
	my $values_hashref = $self->{'e_rp'}->{$chromosome}->{'all'};
	my @keys = sort {$b <=> $a} keys (%$values_hashref);
	my $size = scalar(@keys);
	my $value = 0;
	for(my $i = 0; $i < $size; $i++) {
		if ($keys[$i] == $distance_value) {
			$value = $values_hashref->{$keys[$i]};
			last;
		} elsif (($i+1) != $size) {
			if ($distance_value < $keys[$i] && $distance_value > $keys[$i+1]) {
				$value = ((($values_hashref->{$keys[$i]}  - $values_hashref->{$keys[$i+1]}) / abs( $keys[$i+1] - $keys[$i])) * ($distance_value - $keys[$i])) + $values_hashref->{$keys[$i]};
				last;
			}
		} else {
			warn("Out of range of expectancy values");	
		}
	}
	return $value;
}

sub get_in_repeat_expectancy_value {
	my $self = shift;
	my $chromosome = shift;
	my $distance_value = shift;
	if (!defined($self->{'e_repeat'})) {
		$self->{'e_repeat'} = $self->{'expectancy_loader'}->get_repeat_expectancies();	
	}
	
	my $values_hashref = $self->{'e_repeat'}->{$chromosome}->{'all'}->{'inrepeat'};
	my @keys = sort {$b <=> $a} keys (%$values_hashref);
	my $size = scalar(@keys);
	my $value = 0;
	for(my $i = 0; $i < $size; $i++) {
		if ($keys[$i] == $distance_value) {
			$value = $values_hashref->{$keys[$i]};
			last;
		} elsif (($i+1) != $size) {
			if ($distance_value < $keys[$i] && $distance_value > $keys[$i+1]) {
				$value = ((($values_hashref->{$keys[$i]}  - $values_hashref->{$keys[$i+1]}) / abs( $keys[$i+1] - $keys[$i])) * ($distance_value - $keys[$i])) + $values_hashref->{$keys[$i]};
				last;
			}
		} else {
			warn("Out of range of expectancy values");	
		}
	}
	return $value;	
}

sub get_short_event_expectancy_value {
	my $self = shift;
	my $chromosome = shift;
	my $distance_value = shift;
	if (!defined($self->{'e_repeat'})) {
		$self->{'e_repeat'} = $self->{'expectancy_loader'}->get_repeat_expectancies();	
	}
	
	my $values_hashref = $self->{'e_repeat'}->{$chromosome}->{'all'}->{'shortevent'};
	my @keys = sort {$b <=> $a} keys (%$values_hashref);
	my $size = scalar(@keys);
	my $value = 0;
	for(my $i = 0; $i < $size; $i++) {
		if ($keys[$i] == $distance_value) {
			$value = $values_hashref->{$keys[$i]};
			last;
		} elsif (($i+1) != $size) {
			if ($distance_value < $keys[$i] && $distance_value > $keys[$i+1]) {
				$value = ((($values_hashref->{$keys[$i]}  - $values_hashref->{$keys[$i+1]}) / abs( $keys[$i+1] - $keys[$i])) * ($distance_value - $keys[$i])) + $values_hashref->{$keys[$i]};
				last;
			}
		} else {
			warn("Out of range of expectancy values");	
		}
	}
	return $value;	
}

sub get_long_event_expectancy_value {
	my $self = shift;
	my $chromosome = shift;
	my $distance_value = shift;
	if (!defined($self->{'e_repeat'})) {
		$self->{'e_repeat'} = $self->{'expectancy_loader'}->get_repeat_expectancies();	
	}
	
	my $values_hashref = $self->{'e_repeat'}->{$chromosome}->{'all'}->{'longevent'};
	my @keys = sort {$b <=> $a} keys (%$values_hashref);
	my $size = scalar(@keys);
	my $value = 0;
	for(my $i = 0; $i < $size; $i++) {
		if ($keys[$i] == $distance_value) {
			$value = $values_hashref->{$keys[$i]};
			last;
		} elsif (($i+1) != $size) {
			if ($distance_value < $keys[$i] && $distance_value > $keys[$i+1]) {
				$value = ((($values_hashref->{$keys[$i]}  - $values_hashref->{$keys[$i+1]}) / abs( $keys[$i+1] - $keys[$i])) * ($distance_value - $keys[$i])) + $values_hashref->{$keys[$i]};
				last;
			}
		} else {
			warn("Out of range of expectancy values");	
		}
	}
	return $value;	
}
