package Chum::Utils::GenomeCoordinateLocator;

use vars qw(@ISA @GCL_PARAMS %OK_FIELD $AUTOLOAD);

###The blast database must be separated by longest possible genomic units (like chromosomes)

use strict;
use Bio::Root::Root;
use Bio::Tools::Run::StandAloneBlast;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );

BEGIN {
	@GCL_PARAMS = qw(blast_db);
	foreach my $attr (@GCL_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
	while (@args) {
		my $attr = shift @args;
		my $value = shift @args;
		$self->$attr($value);
	}

	##SET-UP BLAST
	my $init_factory = Bio::Tools::Run::StandAloneBlast->new(	'database' => 'foo.fa',
									'program' => "wublastn",
									'E' => "1");  ##For some reason this needs to be here

	my @params = (  'database' => $self->{'blast_db'},
			'program' => 'wublastn');
	$self->{'_blastfactory'} = Bio::Tools::Run::StandAloneBlast->new(@params);
	$self->{'_blastfactory'}->executable('blastn');
	$self->{'_blastfactory'}->p('blastn');
	$self->{'_blastfactory'}->E('100');
	$self->{'_blastfactory'}->hspsepsmax('5000');
	$self->{'_blastfactory'}->sort_by_highscore(1);

	return $self;
}

sub AUTOLOAD {
	my $self = shift;
	my $attr = $AUTOLOAD;
	$attr =~ s/.*:://;

	$self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
	$self->{$attr} = shift if @_;
	return $self->{$attr};
}

sub locate_sequence {
	my $self = shift;
	my $id = shift;
	my $sequence = shift;

	my $seqobj = Bio::Seq->new( 	-id => $id,
					-seq => $sequence );

	my $hits = $self->_run_blast($seqobj);
	return $hits;
}

sub _run_blast {
	my $self = shift;
	my $input = shift;  #Bio::Seq object

	my $blast_report = $self->{'_blastfactory'}->wublast($input);
	my %blast_feature;
	while(my $result = $blast_report->next_result) {
		while (my $hit = $result->next_hit ) {
			while(my $hsp = $hit->next_hsp) {
				my %t_blast_feature;
				$t_blast_feature{'evalue'} = $hsp->evalue;
				$t_blast_feature{'hstart'} = $hsp->hit->start;
				$t_blast_feature{'hend'} = $hsp->hit->end;
				$t_blast_feature{'hseqid'} = $hsp->hit->seq_id;
				$t_blast_feature{'hstrand'} = $hsp->hit->strand;
				$t_blast_feature{'hseq'} = $hsp->hit_string;
				$t_blast_feature{'qstart'} = $hsp->query->start; ##position in query sequence that matches this hit sequence!
				$t_blast_feature{'qend'} = $hsp->query->end;
				$t_blast_feature{'qseqid'} = $hsp->query->seq_id;
				$t_blast_feature{'qstrand'} = $hsp->query->strand;
				$t_blast_feature{'score'} = $hsp->score;
				if (%blast_feature) {
					if ($t_blast_feature{'score'} > $blast_feature{'score'}) {
						my $chromosome = $hsp->hit->seq_id;
						unless ($chromosome =~ /([\S]*)random([\S]*)/g) {	
							%blast_feature = %t_blast_feature;
						}
					}
				}
				else {
					my $chromosome = $hsp->hit->seq_id;
					unless ($chromosome =~ /([\S]*)random([\S]*)/g) {
						%blast_feature = %t_blast_feature;
					}
				}
			}
		}
	}
	my @hit_results = ();
	if (%blast_feature) {
		my $chromosome = %blast_feature->{'hseqid'};
		if ($chromosome =~ /chr([\S]+)/g) { $chromosome = $1; }
		push @hit_results,  {	'chromosome' => $chromosome, 
					'start' => %blast_feature->{'hstart'}, 
					'end' => %blast_feature->{'hend'}, 
					'strand' => %blast_feature->{'qstrand'}, 
					'evalue' => %blast_feature->{'evalue'}, 
					'offset' => %blast_feature->{'qstart'}};		
	}
	return \@hit_results;
}

1;
