#
# Chum::Utils::Math
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Utils::Math - Math helper functions

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Utils::Math;

use vars qw(@ISA @MATH_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @MATH_PARAMS = qw();
        foreach my $attr (@MATH_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub mean() {
	my $self = shift;
	my $results_arrayref = shift;
	my $total = 0;
	foreach my $result (@$results_arrayref) {
		$total += $result;
	}
	return $total / scalar(@$results_arrayref);
}

sub stdev() {
	my $self = shift;
	my $results_arrayref = shift;
	my $mean = shift;
	if (!defined($mean)) {
		$mean = $self->mean($results_arrayref);
	}
	my $total = 0;
	foreach my $result (@$results_arrayref) {
		$total += ($result - $mean) * ($result - $mean);
	}
	return sqrt($total / scalar(@$results_arrayref));
}