#!/usr/bin/perl

use strict;
use Chum::Analysis::Local::CoexpressionScanner;

###############################################LOAD INPUT PARAMETERS
my $ensembl_gene_id;
my $output_file;
my $mapping_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-g/) {
        $ensembl_gene_id = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-o/) {
        $output_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-m/) {
        $mapping_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}

###############################################RUN ANALYSIS
if (defined($mapping_file)) { &load_all($mapping_file); }
else {
	if (!defined($output_file)) { die("No output file defined!");}

	my $cs = new Chum::Analysis::Local::CoexpressionScanner( 
			'coexpression_file' => "/home/obig/Projects/coexpression_db/tmm/ensembl_mapped/coexpression.tmm_links_all_human_ENS30_35c.new.txt",
			'opossum_install_dir' => "/home/smontgom/oPOSSUM/oPOSSUM");
	my @coexpressed_genes = @{$cs->_get_coexpressed_genes($ensembl_gene_id, 3)};
	print STDERR "Found " . scalar(@coexpressed_genes) . " coexpressed genes\n";
	my ($factors_arrayref, $modifier_scores_hashref, $excluded_genes_arrayref) = $cs->_get_factors_for_coexpressed_genes(\@coexpressed_genes);
	open (OUT, ">" . $output_file) || die("Cannot open output file");
	foreach my $factor (@$factors_arrayref) {
		print OUT $factor . "\t" . $modifier_scores_hashref->{$factor} . "\n";
	}
	close(OUT);
}
exit(0);

##################################################SUBROUTINES
sub load_all() {
	my $mapping_file = shift;
	unless(-e $mapping_file) {die("Mapping file does not exist");}

	open (MAPPING, $mapping_file ) || die ("ERROR: Cannot open mapping file");
	my %ids;
	while (my $line = <MAPPING>) {
		chomp($line);
		if ($line =~ /ORegAnno:(OREG[0-9]{7})[\s\S]+EnsEMBL:([\S]+)[\s]*/gi)  {
			my $oregid = $1;
			my $ensid = $2;
			%ids->{$ensid} = 1;
		}
	}
	close(MAPPING);

	for my $key (keys %ids) {
		print "perl58 /home/smontgom/sNPeGEn/scripts/analysis/buildCoexpressionMatrixFile.pl -g " . $key . " -o /home/smontgom/sNPeGEn/allstacks/coexpression_matrices/" . $key . ".matrix &\n";
	}
}