#!/usr/bin/perl

use strict;

################################################# IMPORTS
use Bio::EnsEMBL::Registry;
use Chum::Utils::RSNPAnalysisXMLFileReader;
use Data::Dumper;

##TRANSFAC ANALYSIS IMPORTS
##
use Chum::Analysis::Local::MatrixScanner;
use Chum::Result::Interpretter::MatrixScannerInterpretter;
require Chum::DB::Transfac;

##WEEDER ANALYSIS IMPORTS
##
use Chum::Analysis::Local::Weeder;
use Chum::Result::Interpretter::WeederInterpretter;

##MOTIFSAMPLER ANALYSIS IMPORTS
##
use Chum::Analysis::Local::Motifsampler;
use Chum::Result::Interpretter::MotifsamplerInterpretter;

##BANANA ANALYSIS IMPORTS##
##
use Chum::Analysis::Local::Banana;
use Chum::Result::Interpretter::BananaInterpretter;

##DAN ANALYSIS IMPORTS##
##
use Chum::Analysis::Local::Dan;
use Chum::Result::Interpretter::DanInterpretter;

##UTILITY IMPORTS##
##
use Chum::Utils::GeneFinder;
use Chum::Utils::Kaks;
use Chum::Utils::RepeatContentFinder;
use Chum::Utils::AlleleFrequencyFinder;

##SPECIAL DATA ANALYSES IMPORTS##
##
require Chum::DB::Jaspar;
use Chum::Analysis::ExtData::CpGIsland;
use Chum::Analysis::ExtData::NhgriDNaseISite;
use Chum::Analysis::ExtData::NobleDNaseISite;
use Chum::Analysis::ExtData::PhastCons;
use Chum::Analysis::ExtData::RegulatoryPotential;

###############################################CONSTANTS
my $background_file = "/home/smontgom/motifsampler/human_promoters.bg";

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
my $output_file;
my $test_line;
my $local = 0;
my $ens_gene_id;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        	$rsnpa_xml_file = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-o/) {
			$output_file = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-l/) {
        	$local = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-x/) {
        	$test_line = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-g/) {
        	$ens_gene_id = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }
if (!defined($output_file)) { die("ERROR: Output file not defined: -o\n"); }
if (!defined($ens_gene_id)) { die("ERROR: Target EnsEMBL gene required -g"); }

Bio::EnsEMBL::Registry->load_all();

###############################################LOAD PREVIOUS RESULTS IF THE EXIST
my ($results1, $results2, $results3, $results4, $results5, $results6, $results7, $results8, $results9, $results10, $results11, $results12, $results13, $results14, $results15, $results16, $results17);
if (-e $output_file) {
	open(INPUT, $output_file) || die("Cannot open existing output file for intput\n");
	my $run_number;
	my $run_line;
	my $run_status;
	foreach my $line (<INPUT>) {
		chomp($line);
		if ($line =~ /[\S]{1}TEST([0-9]+):[\S]+:([\S]+)/g || $line =~ /[\S]{1}(DONE)/g) {
			if (defined($run_number)) { 
				if ($run_number == 1) { $results1 = $run_line; }
				elsif ($run_number == 2) { $results2 = $run_line; }
				elsif ($run_number == 3) { $results3 = $run_line; }
				elsif ($run_number == 4) { $results4 = $run_line; }
				elsif ($run_number == 5) { $results5 = $run_line; }
				elsif ($run_number == 6) { $results6 = $run_line; }
				elsif ($run_number == 7) { $results7 = $run_line; }
				elsif ($run_number == 8) { $results8 = $run_line; }
				elsif ($run_number == 9) { $results9 = $run_line; }
				elsif ($run_number == 10) { $results10 = $run_line; }
				elsif ($run_number == 11) { $results11 = $run_line; }
				elsif ($run_number == 12) { $results12 = $run_line; }
				elsif ($run_number == 13) { $results13 = $run_line; }
				elsif ($run_number == 14) { $results14 = $run_line; }
				elsif ($run_number == 15) { $results15 = $run_line; }
				elsif ($run_number == 16) { $results16 = $run_line; }
				elsif ($run_number == 17) { $results17 = $run_line; }
			}
			if ($run_number eq "DONE") { last; }
			$run_number = $1;
			$run_line = $line . "\n";
			$run_status = $2;
		} elsif (defined($run_number)) {
			$run_line .= $line . "\n";
		}
	}
	print STDERR "LOADED\n" . $results1 . $results2 . $results3 . $results4 . $results5 . $results6 . $results7 . $results8 . $results9 . $results10 . $results11 . $results12 . $results13 . $results14 . $results15 . $results16 . $results17;
	close(INPUT);
}

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
open (OUTPUT, ">" . $output_file) || die("Cannot open output file\n");

###############################################RUN ALL TESTS############################
my @tokens = split(':', $test_line);
my %IGNORE_TOKENS;
foreach my $token (@tokens) {
	%IGNORE_TOKENS->{$token} = 1;
}

########################TEST 1: TRANSFAC############################################
if (!defined(%IGNORE_TOKENS->{"1"})) {
	print STDERR "Running TEST1:TRANSFAC\n";
	$results1 = "";
	eval {
		my $flank_length = 10;
		my $stack = $rsnpa_obj->get_flanked_sequence_stack_obj($flank_length, 'hosa');
		my $transfac = new Chum::Analysis::Local::Transfac('transfac_dir' => "/home/pubseq/databases/TransFac/current/www/biobase/transfac/7.2/data");
		my $resultset = $transfac->run_variation_analysis($stack);
		my $tinterpretter = new Chum::Result::Interpretter::TransfacInterpretter();
		my $score = $tinterpretter->interpret($resultset);
		$results1 = $score;
	};
	if (@$) {
		print STDERR "ERROR: " . @$ . "\n";
		$results1 = "#TEST1:TRANSFAC:FAILED\n" . $results1 . "\n";
	} else {
		$results1 = "#TEST1:TRANSFAC:SUCCESS\n" . $results1 . "\n";
	}
}
#######################TEST2: WEEDER################################################
if (!defined(%IGNORE_TOKENS->{"2"})) {
	print STDERR "Running TEST2:WEEDER\n";
	$results2 = "";
	eval {
		my $stack = $rsnpa_obj->get_sequence_stack_obj('hosa');
		my $resultset;
		if (!$local) {
				my $weeder = new Chum::Analysis::Remote::Weeder();
				my @wservices = @{$weeder->get_remote_services};
				my $wrandom_index = rand @wservices;
				my $wservice = @wservices[$wrandom_index];
				print STDERR "Weeder running at " . $wservice->getLocation . "\n";
				if (scalar(@wservices) > 0) {
						my @wparameters = @{$weeder->get_input_parameters($wservice)};
						foreach my $wparameter (@wparameters) {
								if ($wparameter->name eq "t") {
										$wparameter->{'value'} = 500;
								} elsif ($wparameter->name eq "s") {
										$wparameter->{'value'} = "true";
								} elsif ($wparameter->name eq "m") {
										$wparameter->{'value'} = "true";
								}
						}
						$resultset = $weeder->run_variation_analysis($wservice, $stack, \@wparameters);
				} else {
						die "ERROR: Not running weeder\n";
				}
		} else {
				my $weeder = new Chum::Analysis::Local::Weeder();
				$weeder->{'binary'} = "/home/smontgom/chinook-internal-server/services/weeder/Weeder1.0.1/weederlaunchersm.pl";
				my $meta_input_hashref = $weeder->get_meta_input_hashref;
				$meta_input_hashref->{'-o'} = "HS";
				$meta_input_hashref->{'-r'} = "small";
				$meta_input_hashref->{'-s'} = "";
				$meta_input_hashref->{'-m'} = "";
				$meta_input_hashref->{'-t'} = "500";
				$resultset = $weeder->run_variation_analysis($stack, $meta_input_hashref);
		}
		if(defined($resultset)) {
				my $winterpretter = new Chum::Result::Interpretter::WeederInterpretter();
				my $variation_name = $stack->get_variation_name;
				my $score = $winterpretter->interpret($resultset);
				my $maxscore = $winterpretter->maximum($resultset);
				print STDERR $stack->reference_sequence->sequence->seq . "\n";
				foreach my $variant_sequence (@{$stack->variant_sequences}) {
						print STDERR $variant_sequence->sequence->seq . "\n";
				}
				print STDERR "CHUMRES:\t" . $variation_name . "\tWEEDER\t" . $score .  "\t" . $stack->get_sequence_ids_string . "\n";
				$results2 = "VAR:" . $score . "\nMAX:" . $maxscore;
		} else {
				die "ERROR: Could not get results\n";
		}
	};
	if (@$) {
			print STDERR "ERROR: " . @$ . "\n";
			$results2 = "#TEST2:WEEDER:FAILED\n" . $results2 . "\n";
	} else {
			$results2 = "#TEST2:WEEDER:SUCCESS\n" . $results2 . "\n";
	}
}

##############################TEST3: MOTIFSAMPLER################################################
if (!defined(%IGNORE_TOKENS->{"3"})) {
	print STDERR "Running TEST3:MOTIFSAMPLER\n";
	$results3 = "";
	eval {
		my $flank_length = 10;
		my $stack = $rsnpa_obj->get_flanked_sequence_stack_obj($flank_length, 'hosa');

		my $resultset;
		if (!$local) {
				my $motifsampler = new Chum::Analysis::Remote::Motifsampler();
				my @mservices = @{$motifsampler->get_remote_services};
				my $mrandom_index = rand @mservices;
				my $mservice = @mservices[$mrandom_index];
				print STDERR "Motifsampler running at " . $mservice->getLocation . "\n";
				if (scalar(@mservices) > 0) {
						my @mparameters = @{$motifsampler->get_input_parameters($mservice)};
						foreach my $mparameter (@mparameters) {
								if ($mparameter->name eq "p") {
										$mparameter->{'value'} = 0.95;
								}
						}
						my $meta_input_hashref = $motifsampler->get_meta_input_hashref;
						$meta_input_hashref->{'background_file'} = $background_file;

						$resultset = $motifsampler->run_variation_analysis($mservice, $stack, \@mparameters, $meta_input_hashref);
				} else {
						die "ERROR: Not running Motifsampler\n";
				}
		} else {
				my $motifsampler = new Chum::Analysis::Local::Motifsampler();
				$motifsampler->{'binary'} = "/home/smontgom/motifsampler/MotifSampler";
				my $meta_input_hashref = $motifsampler->get_meta_input_hashref;
				$meta_input_hashref->{'background_file'} = $background_file;
				$meta_input_hashref->{'-p'} = "0.95";
				$resultset = $motifsampler->run_variation_analysis($stack, $meta_input_hashref);
		}
		if (defined($resultset)) {
				my $minterpretter = new Chum::Result::Interpretter::MotifsamplerInterpretter();
				my $score = $minterpretter->interpret($resultset);
				my $maxscore = $minterpretter->maximum($resultset);
				my $variation_name = $stack->get_variation_name;
				print STDERR "CHUMRES:\t" . $variation_name . "\tMOTIFSAMPLER\t" . $score .  "\t" . $stack->get_sequence_ids_string . "\n";
				$results3 = "VAR:" . $score . "\nMAX:" . $maxscore;
		} else {
				die "ERROR: Could not get results\n";
		}
	};
	if (@$) {
		print STDERR "ERROR: " . @$ . "\n";
		$results3 = "#TEST3:MOTIFSAMPLER:FAILED\n" . $results3 . "\n";
	} else {
		$results3 = "#TEST3:MOTIFSAMPLER:SUCCESS\n" . $results3 . "\n";
	}
}

##############################TEST4: BANANA################################################
if (!defined(%IGNORE_TOKENS->{"4"})) {
	print STDERR "Running TEST4:BANANA\n";
	$results4 = "";
	eval {	
		my $stack = $rsnpa_obj->get_sequence_stack_obj('hosa');
		my $resultset;  
		if (!$local) {  
				my $banana = new Chum::Analysis::Remote::Banana();
				my @bservices = @{$banana->get_remote_services};
				my $brandom_index = rand @bservices;
				my $bservice = @bservices[$brandom_index];
				print STDERR "Banana running at " . $bservice->getLocation . "\n";
				if (scalar(@bservices) > 0) {
						my @bparameters = @{$banana->get_input_parameters($bservice)};
						$resultset = $banana->run_variation_analysis($bservice, $stack, \@bparameters);
				} else {
						die "ERROR: Not running banana\n";
				}
		} else {
			print STDERR "Running banana local\n";
				my $banana = new Chum::Analysis::Local::Banana();
				$resultset = $banana->run_variation_analysis($stack, $banana->get_meta_input_hashref);
		}

		if (defined($resultset)) {
				my $binterpretter = new Chum::Result::Interpretter::BananaInterpretter();
				my $bscore = $binterpretter->interpret($resultset, 'bend');
				print STDERR "BEND SCORE: " . $bscore . "\n";
			my $cscore = $binterpretter->interpret($resultset, 'curve');		
			print STDERR "CURVE SCORE: " . $cscore . "\n";
			$results4 = "BEND:" . $bscore . "\nCURVE:" . $cscore;
		} else {
				die "ERROR: No results generated\n";
		}
	};
	if (@$) {
		print STDERR "ERROR: " . @$ . "\n";
		$results4 = "#TEST4:BANANA:FAILED\n" . $results4 . "\n";
	} else {
		$results4 = "#TEST4:BANANA:SUCCESS\n" . $results4 . "\n";;
	}
}

##############################TEST5: DAN################################################
if (!defined(%IGNORE_TOKENS->{"5"})) {
	print STDERR "Running TEST5:DAN\n";
	$results5 = "";
	eval {
		my $stack = $rsnpa_obj->get_sequence_stack_obj('hosa');
		my $resultset;
		if (!$local) {
				my $dan = new Chum::Analysis::Remote::Dan();
				my @dservices = @{$dan->get_remote_services};
				my $drandom_index = rand @dservices;
				my $dservice = @dservices[$drandom_index];
				print STDERR "Dan running at " . $dservice->getLocation . "\n";
				if (scalar(@dservices) > 0) {
						my @dparameters = @{$dan->get_input_parameters($dservice)};
						$resultset = $dan->run_variation_analysis($dservice, $stack, \@dparameters);
				} else {
						die "ERROR: Not running dan\n";
				}
		} else {
				my $dan = new Chum::Analysis::Local::Dan();
				$resultset = $dan->run_variation_analysis($stack, $dan->get_meta_input_hashref);
		}
		if (defined($resultset)) {
				my $dinterpretter = new Chum::Result::Interpretter::DanInterpretter();
				my $gscore = $dinterpretter->interpret($resultset, 'gc');
				print STDERR "GC SCORE: " . $gscore . "\n";
			my $tscore = $dinterpretter->interpret($resultset, 'tm');
					print STDERR "TM SCORE: " . $tscore . "\n";
			$results5 = "GC:" . $gscore . "\nTM:" . $tscore;
		} else {
				die "ERROR: No results generated\n";
		}

	};
	if (@$) {
		print STDERR "ERROR: " . @$ . "\n";
		$results5 = "#TEST5:DAN:FAILED\n" . $results5 . "\n";
	} else {
		$results5 = "#TEST5:DAN:SUCCESS\n" . $results5 . "\n";
	}
}

##############################TEST6,7: GENE DISTANCE, KAKS#################################
if (!defined(%IGNORE_TOKENS->{"6,7"})) {
	print STDERR "Running TEST6/7:DISTANCE/KAKS\n";
	$results6 = "";
	$results7 = "";
	eval {
		my $stack = $rsnpa_obj->get_sequence_stack_obj('hosa');
		my $gene_finder = new Chum::Utils::GeneFinder();
		my $kaks = new Chum::Utils::Kaks();

		my $transcript_distance_hashref = $gene_finder->get_ensembl_transcript_distances_to_stack($stack);
		my @gene_names = ();
		if (defined $transcript_distance_hashref) {
				foreach my $key (keys %$transcript_distance_hashref) {
					print STDERR $key . " " . $transcript_distance_hashref->{$key} . "\n";
					my @ids = split(/:/, $key);
					if ($ids[0] eq $ens_gene_id) {
						my $mdistance = abs($transcript_distance_hashref->{$key});
						if ($mdistance == 0) {
							$mdistance = 1;
						} 
						$results6 .= $key . ":" . $transcript_distance_hashref->{$key} . ":" . log($mdistance) . "\n";
						if ($ids[0] ne "unknown") {
								push @gene_names, $ids[0];
						}
					}
				}
				if (scalar(@gene_names) == 0) {
					foreach my $key (keys %$transcript_distance_hashref) {
						print STDERR $key . " " . $transcript_distance_hashref->{$key} . "\n";
						my @ids = split(/:/, $key);
						my $mdistance = abs($transcript_distance_hashref->{$key});
						if ($mdistance == 0) {
							$mdistance = 1;
						} 
						$results6 .= $key . ":" . $transcript_distance_hashref->{$key} . ":" . log($mdistance) . "\n";
						if ($ids[0] ne "unknown") {
								push @gene_names, $ids[0];
						}
					}
				}
		}
		my $kaks_results_hashref = $kaks->get_kaks_species_and_distances_for_gene_names_hashref(\@gene_names);
		if (defined($kaks_results_hashref)) {
			foreach my $key (keys %$kaks_results_hashref) {
				my $scores_arrayref = $kaks_results_hashref->{$key};
				if (defined($scores_arrayref)) {
					foreach my $score (@$scores_arrayref) {
						$results7 .= $key . ":" . $score->{'species1'} . ":" . $score->{'species2'} . ":" . $score->{'kaks'} . "\n"; 
					}
				} else {
					$results7 .= $key . ":\n";
				}
			}
		} else {
			$results7 = "\n";
		}
	};
	if (@$) {
		print STDERR "ERROR: " . @$ . "\n";
		$results6 = "#TEST6:DISTANCE:FAILED\n" . @$ . "\n";
		$results7 = "#TEST7:KAKS:FAILED\n" . @$ . "\n";
	} else {
		$results6 = "#TEST6:DISTANCE:SUCCESS\n" . $results6;
		$results7 = "#TEST7:KAKS:SUCCESS\n" . $results7;
	}
}

##############################TEST8: REPEAT CONTENT#################################
if (!defined(%IGNORE_TOKENS->{"8"})) {
	print STDERR "Running TEST8:REPEAT\n";
	$results8 = "";
	eval {
		my $flank_length = 10000;
		my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');
		my $rcf = new Chum::Utils::RepeatContentFinder();
		my $repeat_content = $rcf->get_repeat_content($stack_object, $flank_length);
		if (defined($repeat_content)) {
			$results8 .= "LENGTH:" . $repeat_content->{'length'} . "\n";
			$results8 .= "VAR_IN_REPEAT:" . $repeat_content->{'in_repeat'} . "\n";
			$results8 .= "LONG_EVENTS:" . $repeat_content->{'events_long'} . "\n";
			$results8 .= "SHORT_EVENTS:" . $repeat_content->{'events_short'} . "\n";
			my $repeat_types_hashref = $repeat_content->{'repeats'};
			foreach my $key (keys %$repeat_types_hashref) {
				$results8 .= $key . ":" . $repeat_types_hashref->{$key} . "\n";			
			}		
		} else {
			$results8 = "\n";
		}
	};
	if (@$) {
			print STDERR "ERROR: " . @$ . "\n";
			$results8 = "#TEST8:REPEAT:FAILED\n";
	} else {
			$results8 = "#TEST8:REPEAT:SUCESS\n" . $results8;
	}
}

##############################TEST9: ALLELE FREQUENCY################################
if (!defined(%IGNORE_TOKENS->{"9"})) {
	print STDERR "Running TEST9:ALLELEFREQ\n";
	$results9 = "";
	eval {
		my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');
		my $aff = new Chum::Utils::AlleleFrequencyFinder();
		my $allele_freqs = $aff->get_allele_frequencies($stack_object);
		if (defined($allele_freqs)) {
			foreach my $allele (@$allele_freqs) {
				$results9 .= $allele->{'allele'} . ":" . $allele->{'frequency'} . "\n";
			}
		} else {
			$results9 = "\n";
		}
	};
	if (@$) {
		print STDERR "ERROR: " . @$ . "\n";
		$results9 = "#TEST9:ALLELEFREQ:FAILED\n";
	} else {
		$results9 = "#TEST9:ALLELEFREQ:SUCCESS\n" . $results9;
	}
}

############################TEST10: MONTHAM##########################################
if (!defined(%IGNORE_TOKENS->{"10"})) {
	print STDERR "Running TEST10:MONTHAM\n";
	$results10 = "";
	eval {
		my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');
		my $montham = new Chum::Analysis::Montham::Montham();
		my $results_hashref = $montham->montham($stack_object);
		if (defined($results_hashref)) {
			foreach my $key1 (keys %$results_hashref) {
				my $var_hashref = $results_hashref->{$key1};
				foreach my $key2 (keys %$var_hashref) {
					$results10 .= $key1 . ":" . $key2 . ":" . $results_hashref->{$key1}->{$key2} . "\n";
				}
			}
		} else {
			$results10 = "\n";
		}
	};
	if (@$) {
		print STDERR "ERROR: " . @$ . "\n";
		$results10 = "#TEST10:MONTHAM:FAILED\n";
	} else {
		$results10 = "#TEST10:MONTHAM:SUCCESS\n" . $results10;
	}
}

############################TEST11: TRANSITION/TRANSVERSION##########################################
if (!defined(%IGNORE_TOKENS->{"11"})) {
	print STDERR "Running TEST11:TRANSITION-TRANSVERSION\n";
	$results11 = "";
	eval {
		my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');
		my $allele_ref = lc($stack_object->reference_sequence->allele_sequence->seq);
		if ((length $allele_ref) == 1) {
			my @variant_sequences = @{$stack_object->variant_sequences};
			my $allele_var;
			foreach my $variant_sequence (@variant_sequences) {
				my $av = $variant_sequence->allele_sequence->seq;
				if ((length $av) == 1) {
					$allele_var = lc($av);
					last;
				}
			}
			if (($allele_ref eq 'a' && $allele_var eq 'g') || ($allele_var eq 'a' && $allele_ref eq 'g')) { $results11 = "TRANSITION"; }
			elsif (($allele_ref eq 't' && $allele_var eq 'c') || ($allele_var eq 't' && $allele_ref eq 'c')) { $results11 = "TRANSITION"; }
			elsif (($allele_ref eq 'a' && $allele_var eq 'c') || ($allele_var eq 'a' && $allele_ref eq 'c')) { $results11 = "TRANSVERSION"; }
			elsif (($allele_ref eq 'a' && $allele_var eq 't') || ($allele_var eq 'a' && $allele_ref eq 't')) { $results11 = "TRANSVERSION"; }
			elsif (($allele_ref eq 'g' && $allele_var eq 'c') || ($allele_var eq 'g' && $allele_ref eq 'c')) { $results11 = "TRANSVERSION"; }
			elsif (($allele_ref eq 'g' && $allele_var eq 't') || ($allele_var eq 'g' && $allele_ref eq 't')) { $results11 = "TRANSVERSION"; }
			else {
				#do nothing
			}
		} 
		$results11 .= "\n";
	};
	if (@$) {
		print STDERR "ERROR: " . @$ . "\n";
		$results11 = "#TEST11:TRANSITION-TRANSVERSION:FAILED\n";
	} else {
		$results11 = "#TEST11:TRANSITION-TRANSVERSION:SUCCESS\n" . $results11;
	}
}

########################TEST 12: COEXPRESSION-OPTIMIZED JASPAR#######################
if (!defined(%IGNORE_TOKENS->{"12"})) {
	print STDERR "Running TEST12:COEXPRESSION-JASPAR\n";
	$results12 = "";
	eval {
		my $flank_length = 10;
		my $stack = $rsnpa_obj->get_flanked_sequence_stack_obj($flank_length, 'hosa');
		my $jaspar = new Chum::DB::Jaspar();

		unless(-e "/home/smontgom/sNPeGEn/allstacks/coexpression_matrices/" . $ens_gene_id . ".matrix") {
			die("No matrix file exists");
		}
		open(MATRIX, "/home/smontgom/sNPeGEn/allstacks/coexpression_matrices/" . $ens_gene_id . ".matrix");		
		my @factors = ();
		my %modifier_scores;
		foreach my $line (<MATRIX>) {
			chomp($line);
			if ($line =~ /([\S]+)[\s]+([\S]+)/g) {
				push @factors, $1;
				%modifier_scores->{$1} = $2;
			}
		}
		close(MATRIX);

		my $matrixset = $jaspar->get_matrix_set_for_names(\@factors); 
		my $scanner = new Chum::Analysis::Local::Transfac();  
		$scanner->{'_matrix_set'} = $matrixset;
		my $resultset = $scanner->run_variation_analysis($stack);
		my $tinterpretter = new Chum::Result::Interpretter::TransfacInterpretter();
		my $score = $tinterpretter->interpret_with_modifiers($resultset, \%modifier_scores);
		$results12 = $score;
	};
	if (@$) {
		print STDERR "ERROR: " . @$ . "\n";
		$results12 = "#TEST12:COEXPRESSION-JASPAR:FAILED\n" . $results12 . "\n";
	} else {
		$results12 = "#TEST12:COEXPRESSION-JASPAR:SUCCESS\n" . $results12 . "\n";
	}
}

########################TEST 13: CPG ISLANDS#######################
if (!defined(%IGNORE_TOKENS->{"13"})) {
	print STDERR "Running TEST13:CPG\n";
	$results13 = "";
	eval {
		my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');
		my ($chromosome, $start, $end, $strand) = &getHG17Mapping($rsnpa_obj->{'id'});
		if (defined($chromosome) && defined($start) && defined($end) && defined($strand)) {
			my $cpg_detector = new Chum::Analysis::ExtData::CpGIsland();
			$results13 = $cpg_detector->in_cpg($chromosome, ($start + $stack_object->reference_sequence->start), ($start + $stack_object->reference_sequence->end));
		} else {
			die("Cannot find coordinates\n");
		}
	};
	if (@$) {
	    print STDERR "ERROR: " . @$ . "\n";
		$results13 = "#TEST13:CPG:FAILED\n" . $results13 . "\n";
	} else {
		$results13 = "#TEST13:CPG:SUCCESS\n" . $results13 . "\n";
	}
}

########################TEST 14: NHGRI DNASE1 SITE#######################
if (!defined(%IGNORE_TOKENS->{"14"})) {
	print STDERR "Running TEST14:NHGRI-DNASEI\n";
	$results14 = "";
	eval {
		my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');
		my ($chromosome, $start, $end, $strand) = &getHG17Mapping($rsnpa_obj->{'id'});
		if (defined($chromosome) && defined($start) && defined($end) && defined($strand)) {
			my $ndIs_detector = new Chum::Analysis::ExtData::NhgriDNaseISite();
			$results14 = $ndIs_detector->in_dnase1_hs_site($chromosome, ($start + $stack_object->reference_sequence->start), ($start + $stack_object->reference_sequence->end));
		} else {
			die("Cannot find coordinates\n");
		}
	};
	if (@$) {
		print STDERR "ERROR: " . @$ . "\n";
		$results14 = "#TEST14:NHGRI-DNASEI:FAILED\n" . $results14 . "\n";
	} else {
		$results14 = "#TEST14:NHGRI-DNASEI:SUCCESS\n" . $results14 . "\n";
	}
}

########################TEST 15: NOBLE DNASE1 SITE#######################
if (!defined(%IGNORE_TOKENS->{"15"})) {
	print STDERR "Running TEST15:NOBLE-DNASEI\n";
	$results15 = "";
	eval {
		my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');

		my ($chromosome, $start, $end, $strand) = &getHG15Mapping($rsnpa_obj->{'id'});
		if (defined($chromosome) && defined($start) && defined($end) && defined($strand)) {
			my $nodIs_detector = new Chum::Analysis::ExtData::NobleDNaseISite();
			$results15 = $nodIs_detector->in_dnase1_hs_site($chromosome, ($start + $stack_object->reference_sequence->start), ($start + $stack_object->reference_sequence->end));

		} else {
			die("Cannot find coordinates\n");
		}
	};
	if (@$) {
		print STDERR "ERROR: " . @$ . "\n";
		$results15 = "#TEST15:NOBLE-DNASEI:FAILED\n" . $results15 . "\n";
	} else {
		$results15 = "#TEST15:NOBLE-DNASEI:SUCCESS\n" . $results15 . "\n";
	}
}

########################TEST 16: PHAST CONS SCORE#######################
if (!defined(%IGNORE_TOKENS->{"16"})) {
	print STDERR "Running TEST16:PHASTCONS\n";
	$results16 = "";
	eval {
		my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');
		my ($chromosome, $start, $end, $strand) = &getHG17Mapping($rsnpa_obj->{'id'});
		if (defined($chromosome) && defined($start) && defined($end) && defined($strand)) {
			my $pc_detector = new Chum::Analysis::ExtData::PhastCons('phast_cons_dir' => "/home/pubseq/databases/phast_cons/mzPt1Mm5Rn3Cf1Gg2Fr1Dr1");
			$results16 = $pc_detector->average_phast_cons_score($chromosome, ($start + $stack_object->reference_sequence->start), ($start + $stack_object->reference_sequence->end));
		} else {
			die("Cannot find coordinates\n");
		}
	};
	if (@$) {
		print STDERR "ERROR: " . @$ . "\n";
		$results16 = "#TEST16:PHASTCONS:FAILED\n" . $results16 . "\n";
	} else {
		$results16 = "#TEST16:PHASTCONS:SUCCESS\n" . $results16 . "\n";
	}
}

########################TEST 17: REGULATORY POTENTIAL SCORE#######################
if (!defined(%IGNORE_TOKENS->{"17"})) {
	print STDERR "Running TEST17:RP\n";
	$results17 = "";
	eval {
		my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');
		my ($chromosome, $start, $end, $strand) = &getHG17Mapping($rsnpa_obj->{'id'});
		if (defined($chromosome) && defined($start) && defined($end) && defined($strand)) {
			my $rp_detector = new Chum::Analysis::ExtData::RegulatoryPotential('regulatory_potential_dir' => "/home/pubseq/databases/regulatory_potential");
			$results17 = $rp_detector->average_regulatory_potential_score($chromosome, ($start + $stack_object->reference_sequence->start), ($start + $stack_object->reference_sequence->end));

		} else {
			die("Cannot find coordinates\n");
		}
	};
	if (@$) {
		print STDERR "ERROR: " . @$ . "\n";
		$results17 = "#TEST17:RP:FAILED\n" . $results17 . "\n";
	} else {
		$results17 = "#TEST17:RP:SUCCESS\n" . $results17 . "\n";
	}
}

#############################DONE####################################################
print OUTPUT $results1;
print OUTPUT $results2;
print OUTPUT $results3;
print OUTPUT $results4;
print OUTPUT $results5;
print OUTPUT $results6;
print OUTPUT $results7;
print OUTPUT $results8;
print OUTPUT $results9;
print OUTPUT $results10;
print OUTPUT $results11;
print OUTPUT $results12;
print OUTPUT $results13;
print OUTPUT $results14;
print OUTPUT $results15;
print OUTPUT $results16;
print OUTPUT $results17;

print OUTPUT "#DONE";

print STDERR "DONE ANALYSIS\n";
close(OUTPUT);
exit(0);



################################################################SUBROUTINES

sub getHG17Mapping() {
	my $id = shift;
	my $mapping_directory = "/home/smontgom/sNPeGEn/allstacks/mapped/hg17/";
	my $file = $mapping_directory . $id . ".hg17.mapped";
	unless (-e $file) { die("Cannot find mapping for $file\n"); }
	open(FILE, $file) || die("Cannot open mapping file $file\n");
	my $chromosome;
	my $start;
	my $end;
	my $strand;
	while (my $line = <FILE>) {
		chomp($line);
		if ($line =~ /([\S]+)[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([\S]+)/g) {
			$chromosome = $1;
			$start = $2;
			$end = $3;
			$strand = $4;
		}
	}
	close(FILE);
	return $chromosome, $start, $end, $strand;
}

sub getHG15Mapping() {
	my $id = shift;
	my $mapping_directory = "/home/smontgom/sNPeGEn/allstacks/mapped/hg15/";
	my $file = $mapping_directory . $id . ".hg15.mapped";
	unless (-e $file) { die("Cannot find mapping for $file\n"); }
	open(FILE, $file) || die("Cannot open mapping file $file\n");
	my $chromosome;
	my $start;
	my $end;
	my $strand;
	while (my $line = <FILE>) {
		chomp($line);
		if ($line =~ /([\S]+)[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([\S]+)/g) {
			$chromosome = $1;
			$start = $2;
			$end = $3;
			$strand = $4;
		}
	}
	close(FILE);
	return $chromosome, $start, $end, $strand;
}