#!/usr/bin/perl

use strict;
use Data::Dumper;
use Chum::Analysis::Local::CoexpressionScanner;
require Chum::DB::Jaspar;
use Chum::Result::Interpretter::MatrixScannerInterpretter;
use Chum::Utils::GeneFinder;
use Chum::Utils::RSNPAnalysisXMLFileReader;
use Bio::EnsEMBL::Registry;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
my $flank_length = 10;
my $ensembl_gene_id = undef;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnpa_xml_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-w/) {
        $flank_length = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-g/) {
        $ensembl_gene_id = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }

Bio::EnsEMBL::Registry->load_all();

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack = $rsnpa_obj->get_flanked_sequence_stack_obj($flank_length, 'hosa');
my $stack_map = $rsnpa_obj->get_sequence_stack_obj('hosa');

###############################################TRY TO GET RELEVANT ENSEMBL GENE NAME
if (!defined($ensembl_gene_id)) {
    my $stack_map = $rsnpa_obj->get_sequence_stack_obj('hosa');
    my $gene_finder = new Chum::Utils::GeneFinder();
    my $transcript_distance_hashref = $gene_finder->get_ensembl_transcript_distances_to_stack($stack_map);
    print Dumper $transcript_distance_hashref;
    my $closest_gene_name; 
    my $distance;
    if (defined $transcript_distance_hashref) {
	foreach my $key (keys %$transcript_distance_hashref) {
	    my @ids = split(/:/, $key);
	    if ($ids[0] ne "unknown") {
		if (!defined($closest_gene_name)) { $closest_gene_name = $ids[0];  $distance = $transcript_distance_hashref->{$key}; }	
		elsif ($distance > $transcript_distance_hashref->{$key}) {
		    $closest_gene_name = $ids[0];  $distance = $transcript_distance_hashref->{$key}; 
		}
	    }
	}
    }
    print STDERR "Using gene $closest_gene_name\n";
    if (!defined $closest_gene_name) {
	die("No gene found!\n");
    } else {
	$ensembl_gene_id = $closest_gene_name;
    }
}

###############################################RUN COEXPRESSION SCANNER ANALYSIS
my $cs = new Chum::Analysis::Local::CoexpressionScanner( 
		'coexpression_file' => "/home/obig/Projects/coexpression_db/tmm/ensembl_mapped/coexpression.tmm_links_all_human_ENS30_35c.new.txt",
		'opossum_install_dir' => "/home/smontgom/oPOSSUM/oPOSSUM",
		'matrix_db' => new Chum::DB::Jaspar());
my $resultset = $cs->run_variation_analysis($stack, $ensembl_gene_id);
if (defined($resultset)) {
    my $msi = new Chum::Result::Interpretter::MatrixScannerInterpretter('resultset' => $resultset);
    my $score = $msi->interpret();
    print "SCORE: " . $score . "\n";
} else {
    print "SCORE: N\\A\n";
}
