#!/usr/bin/perl

use strict;
use Data::Dumper;
use Chum::Analysis::Local::Dan;
use Chum::Result::Interpretter::DanInterpretter;
use Chum::Utils::RSNPAnalysisXMLFileReader;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
my $flank_length = 100;
my $local = 0;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
       $rsnpa_xml_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-l/) {
        $local = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack = $rsnpa_obj->get_sequence_stack_obj('hosa');
my $resultset;
if (!$local) {
    require Chum::Analysis::Remote::Dan;
    my $dan = new Chum::Analysis::Remote::Dan();
    my @dservices = @{$dan->get_remote_services};
    my $drandom_index = rand @dservices;
    my $dservice = $dservices[$drandom_index];
    print STDERR "Dan running at " . $dservice->getLocation . "\n";
    if (scalar(@dservices) > 0) {
    	my @dparameters = @{$dan->get_input_parameters($dservice)};
    	$resultset = $dan->run_variation_analysis($dservice, $stack, \@dparameters);
    } else {
    	die "ERROR: Not running dan\n";
    }
} else {
    my $dan = new Chum::Analysis::Local::Dan();
    $resultset = $dan->run_variation_analysis($stack);

}

if (defined($resultset)) {
    my $dinterpretter = new Chum::Result::Interpretter::DanInterpretter('resultset' => $resultset);	
    my $variation_name = $stack->get_variation_name;
    my $max_score = $dinterpretter->calculate_maximum_score('tm');
    my $diff_score = $dinterpretter->calculate_maximum_difference('tm');
    print "MAX SCORE: " . $max_score . "\n";
    print "DIFF SCORE: " . $diff_score . "\n";
} else {
    die "ERROR: No results generated\n";
}