#!/usr/bin/perl

use strict;
use Data::Dumper;
use Chum::Analysis::Montham::Montham;
use Chum::Utils::RSNPAnalysisXMLFileReader;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
my $flank_length = 100;
my $local = 0;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnpa_xml_file = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-w/) {
        $flank_length = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-l/) {
        $local = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');

my $montham = new Chum::Analysis::Montham::Montham();
my $results_hashref = $montham->montham($stack_object);

print Dumper $results_hashref;

exit(0);