#!/usr/bin/perl

use strict;

my $coexpression_file = "/home/obig/Projects/coexpression_db/tmm/ensembl_mapped/coexpression.tmm_links_all_human_ENS30_35c.new.txt";
my $ensembl_gene = undef;
for (my $i = 0; $i < @ARGV; $i++) {
	if ($ARGV[$i] =~ /-f/) {
		$coexpression_file = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-g/) {
		$ensembl_gene = $ARGV[++$i];
	} else {
		print STDERR "ERROR: Invalid parameter: " . $ARGV[$i] . "\n";
		exit(1);
	}
}
if (!defined($coexpression_file)) { die("Coexpression input file not defined: use -f\n"); }
unless(-f $coexpression_file) { die("Coexpression input file does not exist: use -f $coexpression_file\n"); }

open(COEXPRESSION_DATA, $coexpression_file) || die("No coexpression data file found");
my %coexpressed_genes = ();
while (my $record = <COEXPRESSION_DATA>) {
	chomp($record);
	if ($record =~ /(ENSG[0-9]+)[\s]+(ENSG[0-9]+)[\s]+([\S]+)/g) {
		my $gene1 = $1;
		my $gene2 = $2;
		my $score = $3;
		if (defined(%coexpressed_genes->{$gene1}->{$gene2})) {
			%coexpressed_genes->{$gene1}->{$gene2} += $score;
		} else {
			%coexpressed_genes->{$gene1}->{$gene2} = $score;
		}

		if (defined(%coexpressed_genes->{$gene2}->{$gene1})) {
			%coexpressed_genes->{$gene2}->{$gene1} += $score;
		} else {
			%coexpressed_genes->{$gene2}->{$gene1} = $score;
		}
	}
}
close(COEXPRESSION_DATA);

if (-p STDIN) {
	while(<STDIN>) {
		my $line = $_;
		chomp ($line);
		if ($line =~ /[\S\s]*EnsEMBL:([\S]+)[\s\S]*/gi) {
			my $gene = $1;
			if (defined(%coexpressed_genes->{$gene})) {
				my $hashref = %coexpressed_genes->{$gene};
				print $gene . "\t" . scalar(keys(%$hashref)) . "\n";
			} else {
				print $gene . "\t0\n";
			}
		} 
	}
} else {
	if (defined($ensembl_gene)) {
		if (defined(%coexpressed_genes->{$ensembl_gene})) {
			my $hashref = %coexpressed_genes->{$ensembl_gene};
			print $ensembl_gene . "\t" . scalar(keys(%$hashref)) . "\n";
		} else {
			print $ensembl_gene . "\t0\n";
		}
	} else {
		die ("No gene defined: use -g");
	}
}