#!/usr/bin/perl

use strict;
use Bio::EnsEMBL::Registry;
use Data::Dumper;

Bio::EnsEMBL::Registry->load_all();

if (-p STDIN) {
	while(<STDIN>) {
		my $line = $_;
		chomp ($line);
		if ($line =~ /EnsEMBL/) {
			print $line . "\n";
		} elsif ($line =~ /[\S\s]*EntrezGene:([0-9]+)[\S\s]*/gi) {
			eval {
				my $ens_gene_id = &getEnsemblGeneId($1);
				print $line . "\tEnsEMBL:" . $ens_gene_id . "\n";
			};
			if ($@) {
				print $line . "\tEnsEMBL:" . "\n";
			}
		} elsif ($line =~ /([0-9]+)/g) {
			eval {
				my $ens_gene_id = &getEnsemblGeneId($1);
				print "EntrezGene:" . $line . "\tEnsEMBL:" . $ens_gene_id . "\n";
			};
			if ($@) {
				print "EntrezGene:" . $line . "\tEnsEMBL:" . "\n";
			}
		}else {
			print STDERR "ERROR: Unrecognized input $line\n";
			exit(1);
		}
	}
} else {
	my $gene_id = undef;
	for (my $i = 0; $i < @ARGV; $i++) {
		if ($ARGV[$i] =~ /-i/) {
			$gene_id = $ARGV[++$i];
		} else {
			print STDERR "ERROR: Invalid parameter: " . $ARGV[$i] . "\n";
			exit(1);
		}
	}
	
	if (!defined($gene_id)) {
		print STDERR "ERROR: A gene id -i must be specified from Entrez Gene\n";
		exit(1);
	} else {
		eval {
			my $ens_gene_id = &getEnsemblGeneId($gene_id);
			print "EntrezGene:" . $gene_id . "\tEnsEMBL:" . $ens_gene_id . "\n";
		};
		if ($@) {
			print "EntrezGene:" . $gene_id . "\tEnsEMBL:" . "\n";
		}
	}
}
exit(0);

sub getEnsemblGeneId() {
	my $gene_id = shift;
	my %gene_names;

	if (defined($gene_id)) {
		my $ga = Bio::EnsEMBL::Registry->get_adaptor("human", "core", "Gene");
		my @genes = @{$ga->fetch_all_by_external_name($gene_id)};
		foreach my $gene (@genes) {
			my @dbentries = @{$gene->get_all_DBLinks()};
			foreach my $dbentry (@dbentries) {
				if ($dbentry eq "EntrezGene:$gene_id") {
					%gene_names->{$gene->stable_id} = 1;
				}
			}
		}
	} 

	my @keys = keys %gene_names;
	if (scalar(@keys) == 1) {
		return @keys[0];
	} else {
		print STDERR "Cannot find a unique mapping for EntrezGene: $gene_id\n";
		die();
	}
}
