#!/usr/bin/perl

use strict;
use DBI;

my $dbh = DBI->connect("DBI:mysql:host=db01.bcgsc.ca;database=oregano",
						"ensembl", "ensembl",
						{PrintError => 0, RaiseError => 1});

if (-p STDIN) {
	while(<STDIN>) {
		my $line = $_;
		chomp ($line);
		if ($line =~ /[\S\s]*ORegAnno:(OREG[0-9]+)[\S\s]*/gi) {
			eval {
				my $gene_id = &getGeneId($1, $dbh);
				if ($gene_id =~ /ENS[\S]+[0-9]+/g) {
					print $line . "\tEnsEMBL:" . $gene_id . "\n";
				} elsif ($gene_id =~ /[0-9]+/g) {
					print $line . "\tEntrezGene:" . $gene_id . "\n";
				}  else {
					die("No gene id found\n");
				}
			};
			if ($@) {
				print $line . "\tEntrezGene:" . "\n";
			}
		} elsif ($line =~ /(OREG[0-9]+)/g) {
			eval {
				my $gene_id = &getGeneId($1, $dbh);
				if ($gene_id =~ /ENS[\S]+[0-9]+/g) {
					print "ORegAnno:" . $line . "\tEnsEMBL:" . $gene_id . "\n";
				} elsif ($gene_id =~ /[0-9]+/g) {
					print "ORegAnno:" . $line . "\tEntrezGene:" . $gene_id . "\n";
				} else {
					die("No gene id found\n");
				}
			};
			if ($@) {
				print "ORegAnno:" . $line . "\tEntrezGene:" . "\n";
			}
		}else {
			print STDERR "ERROR: Unrecognized input $line\n";
			exit(1);
		}
	}
} else {
	my $oreganno_id = undef;
	for (my $i = 0; $i < @ARGV; $i++) {
		if ($ARGV[$i] =~ /-i/) {
			$oreganno_id = $ARGV[++$i];
		} else {
			print STDERR "ERROR: Invalid parameter: " . $ARGV[$i] . "\n";
			exit(1);
		}
	}
	
	if (!defined($oreganno_id)) {
		print STDERR "ERROR: An ORegAnno id -i must be specified i.e. OREG0001872\n";
	} else {
		eval {
			my $gene_id = &getGeneId($oreganno_id, $dbh);
			if ($gene_id =~ /ENS[\S]+[0-9]+/g) {
				print "ORegAnno:" . $oreganno_id . "\tEnsEMBL:" . $gene_id . "\n";
			} elsif ($gene_id =~ /[0-9]+/g) {
				print "ORegAnno:" . $oreganno_id . "\tEntrezGene:" . $gene_id . "\n";
			} else {
				die("No gene id found\n");
			}
		};
		if ($@) {
			print STDERR $@;
			print "ORegAnno:" . $oreganno_id . "\tEntrezGene:" . "\n";
		}
	}
}

$dbh->disconnect();
exit(0);

sub getGeneId() {
	my $id = shift;
	my $dbh = shift;
	my @gene_names = ();

	if (defined($id) && defined($dbh)) {
		my $sth = $dbh->prepare("SELECT gene_id FROM record WHERE stable_id=\"$id\"");
		$sth->execute or die "can't execute query: $sth->errstr";
		while (my @val = $sth->fetchrow_array()) {
			push @gene_names, @val[0];
		}
		$sth->finish();
	}

	if (scalar(@gene_names) == 1) {
		return @gene_names[0];
	} else {
		print STDERR "Cannot find a unique mapping for ORegAnno: $id\n";
		die();
	}
}