#!/usr/bin/perl

use strict;
use Bio::Tools::GFF;
use Data::Dumper;

#######################################################################################################
##LOAD EACH COMMAND-LINE PARAMETER INTO THE APPROPRIATE VARIABLE#######################################
#######################################################################################################
my $mapping_directory;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-d/) {
        $mapping_directory = $ARGV[++$i];
    } else {
        print "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($mapping_directory)) {
        print "\nSome parameters not defined, ";
        print "usage ./createRPolyPipeJobs.pl ";
        print "-d mapping_dir\n";
        exit(1);      
} 

#######################################################################################################
##GET ALL THE MAPPING FILES############################################################################
#######################################################################################################
my @mapping_files = glob($mapping_directory . "/*.mapping");


#######################################################################################################
##PROCESS EACH MAPPING FILE############################################################################
#######################################################################################################

my $i = 1;
foreach my $mapping_file (@mapping_files) {
	my $index = rindex $mapping_file, 'OREG';
	my $lindex = rindex $mapping_file, '.mapping';
	#print $index . "\t" . $lindex . "\n";
	my $id = substr $mapping_file, $index, ($lindex - $index);

	#open gff file, get ensg id annotation, create job, based on annotation
	my $gffio = Bio::Tools::GFF->new(-file => $mapping_file, -gff_version => 2);
	my $feature = $gffio->next_feature();
	if (defined($feature)) {
		my @ensgene = $feature->annotation->get_Annotations('ensGeneId');
		if (defined(@ensgene)) {
			if (scalar(@ensgene) == 1) {
				my $ensgeneid = @ensgene[0];
				print STDERR "Found: " . $ensgeneid . "\t" . $i++ . "\n";

				print "export LD_ASSUME_KERNEL=2.4.1;" . 
					  "export ENSEMBL_REGISTRY=/home/smontgom/perllibs/ensembl/cisred-ensembl-config/ensembl-config;" .
					  "export PERL5LIB=" . $ENV{'PERL5LIB'} . ";" .
					  "perl58 /home/smontgom/sNPeGEn/scripts/thePipe.pl -gene " . $ensgeneid . " -f " . $mapping_file . " > /home/smontgom/sNPeGEn/results/" . $id . ".results\n";

			} else {
				print STDERR $mapping_file . " failed: 1\n";
			}
		} else {
			print STDERR $mapping_file . " failed: 2\n";
		}
	} else {
		print STDERR $mapping_file . " failed: 3\n";
	}
	$gffio->close();
}
