#!/usr/bin/perl

use strict;

my $variation_file;
my $gene_group_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-v/) {
        $variation_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-g/) {
        $gene_group_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($variation_file)) { die("Variation file is not defined"); }
unless(-f $variation_file) { die("Variation file does not exist: " . $variation_file); }
if (!defined($gene_group_file)) { die("Gene group file is not defined"); }
unless(-f $gene_group_file) { die("Gene group file does not exist: " . $gene_group_file); }

my %onlySNPSfromVariationFile;
open(VF, $variation_file) || die("Variation file cannot be opened");
while (my $line = <VF>) {
    chomp($line);
    if ($line =~ /([\S]+)[\s]+[\S]{1}[\s]+[\S]{1}[\s]*/g) {
        $onlySNPSfromVariationFile{$1} = 1;
    }
}
close(VF);

my $error;
my %GENE_GROUP_HASH;
open(GENEGROUP, $gene_group_file) || die("Cannot open gene group file");
while (my $line = <GENEGROUP>) {
    chomp($line);
    if ($line =~ /([\S\s]+):[\s]*([\S\s]*)/g) {
        my $gene = $1;
        my $snp_line = $2;
        my @snps = split /[\s]+/, $snp_line;
        if (defined($GENE_GROUP_HASH{$gene})) {
            $error = "Gene is defined twice in gene group file: " . $gene . "\n";
            last;
        } else {
            $GENE_GROUP_HASH{$gene} = \@snps;
        }
    } else {
        $error = "Undefined line: " . $line . "\n";
        last;
    }
}
close(GENEGROUP);
if (defined($error)) { die($error); }

#########################################################PROCESS GENE GROUP FILE
foreach my $gene (keys %GENE_GROUP_HASH) {
    my @ids = @{$GENE_GROUP_HASH{$gene}};
    my @valid_ids = ();
    foreach my $id (@ids) {
        if (defined($onlySNPSfromVariationFile{$id})) {
            push @valid_ids, $id;
        } else {
            print STDERR "Ignoring " . $id . "\n";   
        }
    }
    if (scalar(@valid_ids) > 0) {
        print $gene . ":\t";
        foreach my $id (@valid_ids) {
            print $id . "\t";   
        }
        print "\n";
    }
}
exit(0);


