#!/usr/bin/perl
use strict;

#-g ../data/pipeline/16_range_testing/group.file.1-2000.txt

my $groups_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-g/) {
        $groups_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($groups_file)) { die("No input group file defined"); }
unless (-e $groups_file) { die("Input group file does not exist"); }

############################################################OPEN GROUP FILE
##
my %GROUPS;
open(GF, $groups_file) || die ("Could not open group file: " . $groups_file);
while (my $line = <GF>) {
    chomp($line);
    if ($line =~ /([\S]+):([\S\s]+)/g) {
	my $transcript_stable_id = $1;
        my $split_line = $2;
        my @tokens = split /[\s]+/, $split_line;
        foreach my $token (@tokens) {
            if ($token =~ /[\S]+/g) {    
                $GROUPS{$token} = $transcript_stable_id;
            }
        }
    }
}
close(GF);

my %positives;
my %neutrals;
foreach my $id (keys %GROUPS) {
    if ($id =~ /OREG[0-9]+/g) {
        if (defined($positives{$id})) {
            warn "Redundancy detected with id: " . $id;
        }
        $positives{$id} = 1;
    } elsif ($id =~ /rs[0-9]+/g) {
        if (defined($neutrals{$id})) {
            warn "Redundancy detected with id: " . $id;
        }
        $neutrals{$id} = 1;
    } else {
        die("Unknown id: " . $id);
    }
}

print "Positives: " . scalar(keys(%positives)) . "\n";
print "Neutrals: " . scalar(keys(%neutrals)) . "\n";

