#!/usr/bin/perl

use strict;
use Chum::Utils::GenomeCoordinateLocator;
use Chum::Utils::RSNPAnalysisXMLFileReader;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
my $output_file;
my $directory;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnpa_xml_file = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-d/) {
        $directory = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-o/) {
        $output_file = $ARGV[++$i];
	} else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (defined($directory)) {
	unless(-d $directory) { die("Directory $directory does not exist"); }
	&loadDirectory($directory);
	exit(0);
}

if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }

#################################################RUN BLAST
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');
my $seq_obj = $stack_object->reference_sequence->sequence;

my $gcl = new Chum::Utils::GenomeCoordinateLocator('blast_db' => 'hg17.fa');
my $hits_arrayref = $gcl->_run_blast($seq_obj);
if (defined($hits_arrayref)) {
	if (scalar(@$hits_arrayref) == 1) {
		my @hits = @$hits_arrayref;
		if (defined($output_file)) {
			open(OUT, ">" . $output_file) || die("Cannot open output file");
			print OUT @hits[0]->{'chromosome'} . "\t" . @hits[0]->{'start'} . "\t" . @hits[0]->{'end'} . "\t" . @hits[0]->{'strand'} . "\n";
			close(OUT);
		} else {
			print @hits[0]->{'chromosome'} . "\t" . @hits[0]->{'start'} . "\t" . @hits[0]->{'end'} . "\t" . @hits[0]->{'strand'} . "\n";
		}
		exit(0);
	}
}
print STDERR "FAILED for $rsnpa_xml_file\n";
die(1);


##############################################SUBROUTINES

sub loadDirectory() {
	my $directory = shift;

	my @files = glob($directory . "/*.xml");
	my @jobs = ();
	foreach my $file (@files) {
		my $id;
		if ($file =~ /(OREG[0-9]{7})/) {
			$id = $1;
		} elsif ($file =~ /(rs[0-9]+)/) {
			$id = $1;
		} else {
			die("Cannot find id for $file");
		}
		my $job = "perl58 /home/smontgom/sNPeGEn/scripts/mapRSNPAnalysisFilesToHGenome.pl -f $file -o /home/smontgom/sNPeGEn/allstacks/mapped/hg17/" . $id . ".hg17.mapped;";
		push @jobs, $job;
	}

	my %superjobs;
	my $index = 0;
	foreach my $job (@jobs) {
		if (defined(%superjobs->{$index})) {
			my $arrayref = %superjobs->{$index};
			push @$arrayref, $job;
			%superjobs->{$index++} = $arrayref;
		} else {
			my @array = ();
			push @array, $job;
			%superjobs->{$index++} = \@array;
		}
		if ($index >= 100) {
			$index = 0;
		}
	}

	for my $key (keys %superjobs) {
		print "export PATH=/home/pubseq/BioSw/wuBLAST_dec_2005:\$PATH;export BLASTDB=/home/smontgom/databases/blast;export PERL5LIB=" . $ENV{'PERL5LIB'} . ";";
		my $arrayref = %superjobs->{$key};
		foreach my $job (@$arrayref) {
			print $job;
		}
		print "\n";
	}
}