#!/usr/bin/perl

use strict;
use Chum::Statistics::AnalysisResult;
use Chum::Neural::Neuron::TransfacNeuron;
use Data::Dumper;

my $test_group_file;
my $positive_group_file;
my $neutral_group_file;
my $function = "r";
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-i/) {
        $test_group_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-p/) {
        $positive_group_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-n/) {
        $neutral_group_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-f/) {
        $function = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}

if ($function eq "t") {
	if (!defined($positive_group_file)) { die("No positives groups file identified\n"); }
	unless (-e $positive_group_file) { die("Positives groups file does not exist\n"); }
	if (!defined($neutral_group_file)) { die("No neutrals groups file identified\n"); }
	unless (-e $neutral_group_file) { die("Neutrals groups file does not exist\n"); }
} elsif ($function eq "r") {
	if (!defined($test_group_file)) { die("No test file identified\n"); }
	unless (-e $test_group_file) { die("Test file does not exist\n"); }
} else {
	die ("Function: -f must be a 't' (train) or 'r' (run)");
}

my $neuron = new Chum::Neural::Neuron::TransfacNeuron();
if ($function eq "t") {
	my @positives = @{&loadFile($positive_group_file)};
	my @neutrals = @{&loadFile($neutral_group_file)};
	$neuron->train(\@positives, \@neutrals);
} elsif ($function eq "r") {
	my @tests =  @{&loadFile($test_group_file)};
	my $pos = 0;
	my $neg = 0;
	foreach my $test (@tests) {
		my ($value, $score) = $neuron->think($test);
		print $test->{'id'} . "\t" . $value . "\t" . $score . "\n";
		if ($value == 1) {
			$pos++;
		} else {
			$neg++;
		}
	}
	print STDERR $pos . "\t" . $neg . "\n";
}

exit(0);

sub loadFile {
	my $file = shift;
	my @results = ();
	open(FILE, $file) || die ("Cannot open $file");
	foreach my $line (<FILE>) {
		chomp($line);
		push @results, new Chum::Statistics::AnalysisResult( 'file' => $line);
	}
	close(FILE);
	return \@results;
}