#!/usr/bin/perl

use strict;
use Bio::Tools::GFF;
use SOAP::Lite;

my $rsnp_gff_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnp_gff_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnp_gff_file)) { die("No rSNP GFF file defined"); }
unless (-e $rsnp_gff_file) { die("Input rSNP GFF file does not exist"); }


############################################################PROCESS SNPS
##
my @OREGANNOIDS;
my $gffio = Bio::Tools::GFF->new(-file => $rsnp_gff_file, -gff_version => 2);
while(my $feature = $gffio->next_feature()) {
    push @OREGANNOIDS, $feature->primary_tag;
}
$gffio->close;

##############################################FETCH FROM OREGANNO

my %PUBLICATIONSHASH;
foreach my $oreganno_id (@OREGANNOIDS) {
    my $osa = SOAP::Lite
            -> uri('OregannoServerImpl')
            -> proxy('http://www.oreganno.org/oregano/soap/',
                      proxy=>['http'=>'http://wwwcache.sanger.ac.uk:3128'],  timeout => 1000);
    my $response = $osa->fetchRecord(SOAP::Data->name(record_stable_id => SOAP::Data->type(string => $oreganno_id)));
    if ($response->fault) {
        die $response->faultstring;
    } else {
        my $record = $response->result;
        $PUBLICATIONSHASH{$record->{'record'}->{'reference'}} = 1;
    }
}

foreach my $key (keys %PUBLICATIONSHASH) {
    print $key . "\n";   
}
print "TOTAL: " . scalar(keys %PUBLICATIONSHASH) . "\n";
exit(0);