#!/usr/bin/perl -w
use strict;

my $gene_groups_file;
my $results_directory;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-g/) {
        $gene_groups_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-d/) {
        $results_directory = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($gene_groups_file)) { die("No gene group file specified");}
unless (-e $gene_groups_file) { die("Gene group file does not exist: " . $gene_groups_file);}
if (!defined($results_directory)) { die("No results directory specified");}
unless (-d $results_directory) { die("Results directory does not exist: " . $results_directory);}

my %RESULT_FILES;
open(GF, $gene_groups_file) || die ("Could not open gene group file: " . $gene_groups_file);
while (my $line = <GF>) {
    chomp($line);
    if ($line =~ /[\S]+:([\S\s]+)/g) {
        my $split_line = $1;
        my @tokens = split /[\s]+/, $split_line;
        foreach my $token (@tokens) {
            if ($token =~ /[\S]+/g) {    
                my $result_file = $results_directory . "/" . $token;
                $RESULT_FILES{$result_file} = 1;
            }
        }
    }
}
close(GF);
foreach my $result_file (sort keys %RESULT_FILES) {
    print $result_file . "\n";
}
exit(0);