#!/usr/bin/perl -w
use strict;

##############################PROCESS INPUTS##############################
my $output_file = undef;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-h/ || $ARGV[$i] =~ /--help/) {
        print STDERR "weederToGFF.pl usage\n";
        print STDERR "\t-f: specify input file\n";
        print STDERR "\tor use pipe redirect. |\n";
        exit(1);
    } elsif ($ARGV[$i] =~ /-o/) {
        $output_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-f/) {
        $i++; ##Ignore parameter for now...
    } else {
        print STDERR "ERROR: Undefined option: " . $ARGV[$i] . "\n";
        exit(1);
    }
}


##############################READ IN INPUT FROM PIPE OR FROM INPUT FILE##############################
my @lines = ();
if (-p STDIN) {
    while(<STDIN>) {
        my $line = $_;
        push @lines, $line; 
    }
} else {
    my $input_file = undef;
    for (my $i = 0; $i < @ARGV; $i++) {
        if ($ARGV[$i] =~ /-f/) {
            $input_file = $ARGV[++$i];
        } 
    }
    if (!defined($input_file) || !(-f $input_file)) {
        print STDERR "ERROR: Input file is undefined or does not exist, -f: " . $input_file . "\n";
        exit(1);    
    }
    open(INPUT, $input_file) || die("Cannot open input file: " . $input_file . "\n");
    while (my $line = <INPUT>) { push @lines, $line; }
    close(INPUT);
}
if (scalar(@lines) == 0) { die("ERROR: Input is empty\n"); }


##############################TRANSFORM OUTPUT INTO GFF##############################
if (defined($output_file)) {
    open(OUT, ">" . $output_file) || die("Cannot open output file: " . $output_file);
}
my %score_hash;
my $current_seqname;
my $current_motif;
my $current_score;
my $rev;
my $new_block = 1;
my $block = 1;
foreach my $line (@lines) {
        chomp($line);
        
        if ($line =~ /^>([\S]+)[\s]*[\S]*/g) {
            $current_seqname = $1;
            $rev = 0; 
        } elsif ($line =~ /^REVCOMP[\s]+>([\S]+)[\s]*[\S]*/g) {
            $current_seqname = $1;
            $rev = 1;
        } elsif ($new_block && $line =~ /^[\s]{1}([ATCGatcgNn]+)[\s]{1}$/g) {
            $current_motif = $1;
            $current_score = $score_hash{$current_motif};
            $new_block = 0;
        } elsif ($line =~ /[0-9]+\)[\s]{1}([ATCG]+)[\s]{1}([0-9]+[\S]{1}[0-9]+)/g) {
            $score_hash{$1} = $2;
        } elsif ($line =~ /[\S\s]{1}([ATCGNatcgn]+)[\S\s]{1}[\s]+position[\s]+([\S]+)[\S]{1}[\s]+[\S]{1}([\S]+)[\S]{1}/g) {
            my $end = $2 + length($1) - 1;
            my $strand = "1";
            if ($rev == 1) { $strand = "-1"; }
            my $result_line = $current_seqname . "\tWEEDER\tpTFBS\t" . $2 . "\t" . $end . "\t" . $current_score . ":" . $3 . "\t" . $strand . "\t.\t" . "Group " . $block . " ; Sequence " . $1 . "\n";
            if (defined($output_file)) {
                print OUT $result_line;
            } else {
                print $result_line;
            }
        } elsif ($line =~ /^[=]+$/g) {
            $new_block = 1;
            $block++;
        }
}
if (defined($output_file)) {
    close(OUT);
}
exit(0);



##############################OUTPUT RESULTS##############################

exit(0);